@@ -2,7 +2,7 @@ import inspect, glob, os, datetime
 
 from ...utils.api import route, Parameter
 
-from ...libs import logs_lib, flask_lib, secure_lib
+from ...libs import logs_lib, secure_lib
 
 from ...models.main import AlphaException
 
@@ -13,7 +13,7 @@ db = core.db
 log = core.get_logger("api")
 
 
-@route("/admin/key", parameters=[Parameter("key")], methods=["POST"])
+@route("/admin/key", parameters=[Parameter("key")], methods=["POST"], admin=True)
 def get_key():
     return secure_lib.get_cry_operation_code(api["key"])
 
@@ -41,7 +41,7 @@ def admin_logs():
     return logs_lib.get_logs(**api.get_parameters())
 
 
-@route("/admin/process")
+@route("/admin/process", admin=True)
 def get_admin_process():
     import psutil
 
