@@ -1,5 +1,5 @@
 from dataclasses import dataclass
-from typing import List
+
 import uuid
 
 from ...models.database.tests import Test
@@ -12,7 +12,7 @@ from .. import tests
 
 from core import core
 
-api = core.api
+API = core.api
 
 
 @route(
@@ -22,9 +22,7 @@ api = core.api
     parameters=[Parameter("value"), Parameter("options", options=["Y", "N"])],
 )
 def api_test():
-    parameters = api.get_parameters()
-    parameters["uuid"] = uuid.uuid4()
-    return parameters
+    return API.gets(added={"uuid": uuid.uuid4()})
 
 
 @route(
@@ -35,11 +33,11 @@ def api_test():
         Parameter("value"),
         Parameter("options", options=["Y", "N", 1, 2]),
         Parameter("list", ptype=list),
-        Parameter("list_str", ptype=List[str]),
-        Parameter("list_int", ptype=List[int]),
-        Parameter("list_object", ptype=List[TestInput]),
-        Parameter("list_db_object", ptype=List[Test]),
-        Parameter("list_float", ptype=List[float]),
+        Parameter("list_str", ptype=list[str]),
+        Parameter("list_int", ptype=list[int]),
+        Parameter("list_object", ptype=list[TestInput]),
+        Parameter("list_db_object", ptype=list[Test]),
+        Parameter("list_float", ptype=list[float]),
         Parameter("list_default", ptype=list, default=[]),
         Parameter("dict", ptype=dict),
         Parameter("dict_default", ptype=dict, default={}),
@@ -53,22 +51,30 @@ def api_test():
         Parameter("like_mode", ptype=str, mode=ParameterMode.LIKE),
         Parameter("start_like_mode", ptype=str, mode=ParameterMode.START_LIKE),
         Parameter("none_mode", ptype=str, mode=ParameterMode.NONE),
+        Parameter("bool", ptype=bool),
     ],
     description="Route used for auto test",
 )
 def api_test_parameters_route():
-    parameters = api.get_parameters()
+    parameters = API.gets()
     return parameters
 
 
-@route("/test/insert",)
+@route(
+    "/test/insert",
+)
 def test_insert():
     return tests.insert()
 
 
+@route("/test/txt", parameters=[Parameter("message")])
+def test_txt_route():
+    return "test\ntest2"
+
+
 @route("/test/html", parameters=[Parameter("message")])
 def html_message():
-    return api.set_html("hello.html", parameters={"message": api.get("message")})
+    return API.set_html("hello.html", parameters={"message": API.get("message")})
 
 
 @route("/test/methods", methods=["GET"])
@@ -98,7 +104,7 @@ def get_method_5():
 
 @route("/test/status", parameters=[Parameter("status"), Parameter("description")])
 def get_status():
-    api.set_status(**api.get_parameters())
+    API.set_status(**API.gets())
 
 
 @route(
@@ -110,4 +116,4 @@ def get_status():
     ],
 )
 def get_exception():
-    raise AlphaException(**api.get_parameters())
+    raise AlphaException(**API.gets())
