@@ -1,15 +1,14 @@
 from ...utils.api import route, Parameter, ParameterMode
 
-from ...libs import logs_lib, flask_lib, database_lib
+from ...libs import logs_lib, database_lib
 
 from ...models.main import AlphaException
 
-from typing import List
 
 from core import core
 
 API = core.api
-db = core.db
+DB = core.db
 log = core.get_logger("api")
 
 
@@ -18,16 +17,28 @@ def get_schemas():
     return database_lib.get_schemas(**API.gets())
 
 
-@route("/database/tables", admin=True)
+@route(
+    "/database/tables",
+    parameters=[
+        Parameter("binds", ptype=list[str]),
+        Parameter("tables", ptype=list[str]),
+    ],
+    admin=True,
+)
 def liste_tables():
-    return database_lib.get_databases_tables_dict(core)
+    return DB.get_tables_models(**API.gets())
+
+
+@route("/database/tables/names", parameters=[Parameter("bind")], admin=True)
+def liste_tables_names():
+    return DB.get_tables_names(**API.gets())
 
 
 @route(
     "/database/create",
     admin=True,
     parameters=[
-        Parameter("schema", required=True),
+        Parameter("bind"),
         Parameter("table", required=True),
         Parameter("drop", ptype=bool),
     ],
@@ -39,26 +50,28 @@ def create_table():
 @route(
     "/database/drop",
     admin=True,
-    parameters=[Parameter("schema", required=True), Parameter("table", required=True)],
+    parameters=[Parameter("bind"), Parameter("table", required=True)],
 )
 def drop_table():
-    dropped = core.drop_table(API.get("schema"), API.get("table"))
-    if dropped:
-        return "table %s dropped" % API.get("table")
+    return core.db.drop_table(**API.gets())
+
+
+DB_INIT_PARAMETERS = [
+    Parameter("binds", ptype=list[str]),
+    Parameter("tables", ptype=list[str]),
+    Parameter("drop", ptype=bool, default=False),
+    Parameter("truncate", ptype=bool, default=False),
+    Parameter("force", ptype=bool, default=False),
+    Parameter("create", ptype=bool, default=False),
+    Parameter("init", ptype=bool, default=False),
+    Parameter("init_views", ptype=bool, default=False),
+]
 
 
 @route(
     "/database/init",
     admin=True,
-    parameters=[
-        Parameter("binds", ptype=List[str]),
-        Parameter("tables", ptype=List[str]),
-        Parameter("drop", ptype=bool, default=False),
-        Parameter("truncate", ptype=bool, default=False),
-        Parameter("force", ptype=bool, default=False),
-        Parameter("create", ptype=bool, default=False),
-        Parameter("init", ptype=bool, default=True),
-    ],
+    parameters=DB_INIT_PARAMETERS,
 )
 def init_all_database():
     database_lib.init_databases(core, **API.gets())
@@ -72,8 +85,8 @@ def get_blocking_queries():
 @route(
     "/table",
     parameters=[
-        Parameter("schema", ptype=str, default="ALPHA"),
-        Parameter("tablename", ptype=str, required=True),
+        Parameter("bind", ptype=str, default="ALPHA"),
+        Parameter("table", ptype=str, required=True),
         Parameter("order_by", ptype=str),
         Parameter("direction", ptype=str),
         Parameter("page_index", ptype=int),
@@ -89,23 +102,23 @@ def get_transactions_history():
 @route(
     "/table/columns",
     parameters=[
-        Parameter("schema", ptype=str, default="ALPHA"),
-        Parameter("tablename", ptype=str, required=True),
+        Parameter("bind", ptype=str, default="ALPHA"),
+        Parameter("table", ptype=str, required=True),
     ],
 )
 def get_table_columns():
-    return database_lib.get_table_columns(**API.gets())
+    return core.db.get_table_columns(**API.gets())
 
 
 @route(
     "/table/model",
     parameters=[
-        Parameter("schema", ptype=str, default="ALPHA"),
-        Parameter("tablename", ptype=str, required=True),
+        Parameter("bind", ptype=str, default="ALPHA"),
+        Parameter("table", ptype=str, required=True),
     ],
 )
 def get_table_model():
-    return database_lib.get_table_model(**API.gets())
+    return core.db.get_table_model(**API.gets()).get_table_model_structure()
 
 
 @route(
@@ -119,3 +132,8 @@ def get_table_model():
 )
 def get_database_whereused():
     return database_lib.whereused(**API.gets())
+
+
+@route("/database/structure")
+def get_database_structure():
+    return database_lib.get_database_structure(**API.gets())
