@@ -1,4 +1,5 @@
-import datetime, os, inspect, re
+import datetime, os, inspect, re, platform
+
 
 from flask import request, send_from_directory
 
@@ -6,6 +7,8 @@ from ...libs import test_lib, database_lib, api_lib, py_lib, files_lib, config_l
 from ...utils.api import route, Parameter
 from ...models.main import AlphaException
 
+from PIL import Image
+
 from core import core, DB, API
 
 LOG = core.get_logger("api")
@@ -28,18 +31,51 @@ def index():
 
 @route("/profil/pic", logged=True)
 def get_profil_pic():
-    file_path = core.config.get("directories/images")
-    API.get_file(file_path, API.get_logged_user().id)
+    user = API.get_logged_user()
+    filename = f"{user.id}.png"
+    directory = core.config.get("directories/profils_pics")
+
+    API.get_file(directory, filename)
 
 
-@route("/files/upload", logged=True, methods=["POST"])
+def save_profil_pic(file):
+    user = API.get_logged_user()
+    directory = core.config.get("directories/profils_pics")
+    img = Image.open(file)
+    img = img.convert("RGBA")  # Convertir en format PNG
+
+    # Calcul du nouveau rapport d'aspect
+    base_width, base_height = img.size
+    aspect_ratio = base_width / base_height
+
+    if base_width > base_height:
+        new_width = 100
+        new_height = int(new_width / aspect_ratio)
+    else:
+        new_height = 100
+        new_width = int(new_height * aspect_ratio)
+
+    # Redimensionnement de l'image
+    img_resized = img.resize((new_width, new_height), Image.ANTIALIAS)
+
+    # Création d'une nouvelle image avec fond transparent
+    output = Image.new("RGBA", (100, 100), (0, 0, 0, 0))
+    output.paste(img_resized, ((100 - new_width) // 2, (100 - new_height) // 2))
+
+    # Enregistrez l'image avec un nouveau nom contenant l'ID utilisateur
+    filename = f"{user.id}.png"
+    os.makedirs(directory, exist_ok=True)
+    output.save(os.path.join(directory, filename), "PNG")
+
+
+@route("/profil/pic", logged=True, methods=["POST"])
 def upload_file():
-    uploaded_file = request.files["file"]
-    ext = uploaded_file.filename.split(".")[1]
-    filename = str(API.get_logged_user().id) + "." + ext
+    file = request.files["file"]
 
-    file_path = core.config.get("directories/images")
-    API.set_file(file_path, filename)
+    if file:
+        save_profil_pic(file)
+        return "File uploaded"
+    API.set_error("Failed to upload file")
 
 
 @route("status", route_log=False)
@@ -53,19 +89,19 @@ def home():
 
     tests = None
     try:
-        tests = test_lib.get_tests_auto(
-            core.config.get("directories/tests", default=[])
-        )
+        tests = test_lib.get_tests_auto(load_from_db=True)
     except Exception as ex:
         LOG.error(ex)
 
-    conda_env = os.popen("conda env list | grep '*'").read().split("*")[0].strip()
+    mode_color = (
+        "#e74c3c"
+        if core.config.configuration == "prod"
+        else ("#0270D7" if core.config.configuration == "dev" else "#2ecc71")
+    )
 
     parameters = {
         "mode": core.config.configuration,
-        "mode_color": "#e74c3c"
-        if core.config.configuration == "prod"
-        else ("#0270D7" if core.config.configuration == "dev" else "#2ecc71"),
+        "mode_color": mode_color,
         "title": config.get("templates/home/title"),
         "description": config.get("templates/home/description"),
         "year": datetime.datetime.now().year,
@@ -77,16 +113,15 @@ def home():
         "compagny_website": config.get("parameters/compagny_website"),
         "dashboard": config.get("dashboard/dashboard/active"),
         "tests": tests,
-        "databases": database_lib.get_databases_tables_dict(core),
         "date": datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S"),
-        "conda_env": conda_env,
+        "conda_env": os.environ.get("CONDA_DEFAULT_ENV"),
         "alphaz_version": "",
     }
     API.set_html("home.html", parameters=parameters)
 
 
 @route("doc", mode="html")
-def doc():
+def doc_route():
     try:
         import markdown2
     except:
@@ -158,18 +193,25 @@ def get_module_code():
     return {"version": module.__version__}
 
 
-@route("modules", parameters=[Parameter("name"), Parameter("url"), Parameter("mode")])
+@route(
+    "/modules",
+    parameters=[
+        Parameter("name"),
+        Parameter("url"),
+        Parameter("mode", default="dict", options=["dict", "requirements"]),
+    ],
+)
 def get_modules():
     return py_lib.get_modules_infos(**API.gets())
 
 
-@route("cache")
+@route("/cache")
 def get_cache_infos():
     return files_lib.get_size(API.conf.get("directories/cache"), with_unit=True)
 
 
 @route(
-    "configurations",
+    "/configurations",
     parameters=[Parameter("config_path"), Parameter("name"), Parameter("path")],
     admin=True,
 )
@@ -206,13 +248,13 @@ def get_dataclasses_mappings():
     return {x: y.to_json() for x, y in DATACLASS_AUTO_MAP_MATCHS.items()}
 
 
-@route("configs", admin=True)
+@route("/configs", admin=True)
 def get_configs():
     return config_lib.get_configs()
 
 
 @route(
-    "configs/get",
+    "/configs/get",
     parameters=[
         Parameter("keys", ptype=list, required=True),
         Parameter("config"),
@@ -220,3 +262,20 @@ def get_configs():
 )
 def get_configs_key():
     return config_lib.get_configs_key(**API.get_parameters())
+
+
+@route(
+    "/mail",
+    parameters=[
+        Parameter("name", required=True),
+        Parameter("parameters", ptype=list[dict]),
+    ],
+    admin=True,
+    methods=["POST"],
+)
+def mail_me():
+    from ...libs import mail_lib
+
+    mail_lib.send_mail(
+        mail_config=API["name"], parameters_list=API["parameters"], force=True
+    )
