@@ -1,4 +1,3 @@
-from typing import List
 from ....utils.api import route, api, Parameter, ParameterMode
 from ....models.database.users_definitions import Role
 from ....libs.user_management import role_management_lib
@@ -41,7 +40,9 @@ def get_roles_names():
 @route(
     "user-mgt/role",
     logged=True,
-    parameters=[Parameter("name", required=True),],
+    parameters=[
+        Parameter("name", required=True),
+    ],
     category=CATEGORY,
 )
 def get_role():
@@ -51,7 +52,9 @@ def get_role():
 @route(
     "user-mgt/role",
     logged=True,
-    parameters=[Parameter("role", required=True, ptype=Role),],
+    parameters=[
+        Parameter("role", required=True, ptype=Role),
+    ],
     methods=["POST"],
     category=CATEGORY,
 )
@@ -62,7 +65,9 @@ def create_role():
 @route(
     "user-mgt/role",
     logged=True,
-    parameters=[Parameter("role", required=True, ptype=Role),],
+    parameters=[
+        Parameter("role", required=True, ptype=Role),
+    ],
     methods=["PUT"],
     category=CATEGORY,
 )
@@ -73,7 +78,9 @@ def edit_role():
 @route(
     "user-mgt/role",
     logged=True,
-    parameters=[Parameter("name", required=True, ptype=str),],
+    parameters=[
+        Parameter("name", required=True, ptype=str),
+    ],
     methods=["DELETE"],
     category=CATEGORY,
 )
@@ -154,7 +161,9 @@ def add_app_role():
 @route(
     "user-mgt/role/application",
     logged=True,
-    parameters=[Parameter("role_name", required=True, ptype=str),],
+    parameters=[
+        Parameter("role_name", required=True, ptype=str),
+    ],
     methods=["DELETE"],
     category=CATEGORY,
 )
