@@ -1,9 +1,9 @@
 from ....utils.api import route, api, Parameter, ParameterMode
 from ....libs.user_management import user_management_lib
-from ....models.database.users_definitions import User
 
 CATEGORY = "user-mgt"
 
+
 @route(
     "user-mgt/users",
     logged=True,
@@ -17,7 +17,8 @@ CATEGORY = "user-mgt"
         Parameter("page_index", ptype=int),
         Parameter("page_size", ptype=int),
     ],
-    methods=["GET"],category=CATEGORY
+    methods=["GET"],
+    category=CATEGORY,
 )
 def get_users():
     return user_management_lib.get_users(**api.get_parameters())
@@ -26,10 +27,9 @@ def get_users():
 @route(
     "user-mgt/user",
     logged=True,
-    parameters=[
-        Parameter("name", required=True),
-    ],
-    methods=["GET"],category=CATEGORY
+    parameters=[Parameter("name", required=True),],
+    methods=["GET"],
+    category=CATEGORY,
 )
 def get_user():
     return user_management_lib.get_user(**api.get_parameters())
@@ -38,10 +38,9 @@ def get_user():
 @route(
     "user-mgt/user",
     logged=True,
-    parameters=[
-        Parameter("uid", required=True, ptype=str),
-    ],
-    methods=["POST"],category=CATEGORY
+    parameters=[Parameter("uid", required=True, ptype=str),],
+    methods=["POST"],
+    category=CATEGORY,
 )
 def add_user():
     return user_management_lib.add_user(**api.get_parameters())
