@@ -1,14 +1,35 @@
-from ...utils.api import ADMIN_USER_ID_PUBLIC, ADMIN_USER_NAME_PUBLIC, route, Parameter
-from ...models.main import AlphaException
-from ..users import users
+# CORE
 from core import core
 
-api = core.api
+API = core.api
+
+# UTILS
+from ...utils.api import ADMIN_USER_ID_PUBLIC, ADMIN_USER_NAME_PUBLIC, route, Parameter
+
+# MODELS
+from ...models.main import AlphaException
+
+# LIBS
+from ...libs import user_lib, secure_lib
+
+from ..users import users
 
 
 @route("user/infos", logged=True)
 def user_infos():
-    return api.get_logged_user()
+    return API.get_logged_user()
+
+
+@route(
+    "/user/data",
+    parameters=[
+        Parameter("value", required=True),
+        Parameter("columns", ptype=list[str], required=True),
+        Parameter("activity", ptype=bool),
+    ],
+)
+def user_data():
+    return user_lib.get_user_data_from_database(**API.gets(), force_local=True)
 
 
 @route(
@@ -22,23 +43,27 @@ def user_infos():
     ],
 )
 def register():
-    if api.get_logged_user() is not None:
+    if API.get_logged_user() is not None:
         raise AlphaException("logged")
-    users.try_register_user(**api.get_parameters())
+    return users.try_register_user(**API.gets())
+
+
+@route("/password", parameters=[Parameter("password")], admin=True)
+def password():
+    return secure_lib.secure_password(**API.gets())
 
 
 @route(
     "/register/validation",
-    methods=["GET"],
+    methods=["POST"],
     parameters=[Parameter("tmp_token", required=True)],
 )
 def register_validation():
-    if api.get_logged_user() is not None:
+    if API.get_logged_user() is not None:
         raise AlphaException("logged")
-    users.confirm_user_registration(**api.get_parameters())
+    users.confirm_user_registration(**API.gets())
 
 
-# LOGIN
 @route(
     "/auth",
     methods=["POST"],
@@ -48,7 +73,7 @@ def register_validation():
     ],
 )
 def login():
-    return users.try_login(**api.get_parameters())
+    return users.try_login(**API.gets())
 
 
 @route(
@@ -58,14 +83,19 @@ def login():
     admin=True,
 )
 def su_login():
-    return users.try_su_login(**api.get_parameters())
+    return users.try_su_login(**API.gets())
 
 
-@route("/ldap/users", parameters=[Parameter("filters", required=True, ptype=str),])
+@route(
+    "/ldap/users",
+    parameters=[
+        Parameter("filters", required=True, ptype=str),
+    ],
+)
 def get_ldap_users():
     from ..users import users, ldap  # TODO: modify
 
-    return ldap.get_ldap_users(**api.get_parameters())
+    return ldap.get_ldap_users(**API.gets())
 
 
 @route(
@@ -77,12 +107,12 @@ def get_ldap_users():
     ],
 )
 def password_lost():
-    if api.get_logged_user() is not None:
+    if API.get_logged_user() is not None:
         raise AlphaException("logged")
-    if api["username"] is None and api.get["mail"] is None:
+    if API["username"] is None and API.get["mail"] is None:
         raise AlphaException("inputs")
     users.ask_password_reset(
-        api["username"] if api.get["mail"] is None else api.get["mail"]
+        API["username"] if API.get["mail"] is None else API.get["mail"]
     )
 
 
@@ -96,9 +126,9 @@ def password_lost():
     ],
 )
 def password_reset_validation():
-    if api.get_logged_user() is not None:
+    if API.get_logged_user() is not None:
         raise AlphaException("logged")
-    users.confirm_user_password_reset(**api.get_parameters())
+    users.confirm_user_password_reset(**API.gets())
 
 
 @route("/logout", cache=False, logged=False, methods=["GET", "POST"])
@@ -121,4 +151,4 @@ def logout_su():
     ],
 )
 def reset_user_password():
-    users.try_reset_password(**api.get_parameters())
+    users.try_reset_password(**API.gets())
