@@ -4,8 +4,10 @@ import datetime
 
 from ..models.database.main_definitions import Test
 from core import core
+
 DB = core.db
 
+
 def insert():
     """insert test.
 
@@ -17,20 +19,13 @@ def insert():
     Returns:
         [type]: [description]
     """
-    test = Test(
-        name_='name',
-        number_=0,
-        text_='text',
-        date_=datetime.datetime.now()
-    )
+    test = Test(name="name", number=0, text="text", date=datetime.datetime.now())
 
     try:
         added_test = DB.add(test)
     except Exception as ex:
         raise ex
 
-    return DB.select(Test, filters=[
-        Test.name_ == "name"
-    ], first=True, order_by=Test.date_.desc())
-
-    
+    return DB.select(
+        Test, filters=[Test.name == "name"], first=True, order_by=Test.date_.desc()
+    )
