@@ -5,7 +5,7 @@ class CONFIGURATION:
     ALPHA_LOGGER_NAME = "alpha"
     ERRORS_LOGGER_NAME = "errors"
     MONITORING_LOGGER_NAME = "monitoring"
-    
+
     API_LOGGER_NAME = "api"
     HTTP_LOGGER_NAME = "http"
 
@@ -15,8 +15,8 @@ class CONFIGURATION:
 
     # Databases
 
-    MAIN_DATABASE_NAME = "main"
-    USERS_DATABASE_NAME = "users"
+    MAIN_DATABASE_NAME = "MAIN"
+    USERS_DATABASE_NAME = "USERS"
 
     # configuration
 
@@ -25,4 +25,6 @@ class CONFIGURATION:
     CONFIGURATION_ENV_NAME = "ALPHA_CONF"
 
     # Debug
-    DEBUG_SCHEMA = False
\ No newline at end of file
+    DEBUG_SCHEMA = False
+
+    SUPER_USER_PERMISSION = "SUPER_USER"
