@@ -1,29 +1,34 @@
-from dataclasses import dataclass, is_dataclass
-import json
-import sys, os, inspect, copy, requests, enum
-import traceback
-from xml.etree.ElementInclude import default_loader
+# MODULES
+import inspect, requests, json
 
+# UTILS
 from ..utils.api import ROUTES
 
+# MODELS
 from ..models.logger import AlphaLogger
 from ..models.api import ApiMethods
 from ..models.main import AlphaException
-from ..models.api._answer import ApiAnswer
+from ..models.api._answer import ApiAnswer, ApiPagination
 
+# LIBS
 from ..libs import dict_lib, json_lib, py_lib
 
+# CORE
+from .. import timer
+
 MODULES = {}
 
 
 def get_api_answer(
     url: str,
     params: dict = {},
-    log: AlphaLogger = None,
+    log: AlphaLogger | None = None,
     no_log: bool = False,
     method: ApiMethods = ApiMethods.GET,
     requester=None,
-) -> ApiAnswer:
+    allow_none: bool = False,
+    token: str | None = None,
+) -> ApiAnswer | None:
     """Get data from api
 
     Args:
@@ -48,10 +53,12 @@ def get_api_answer(
     if hasattr(requests, method_str):
         fct = getattr(requests, method_str)
 
-    params = copy.copy(params)
+    converted_params = {}
     for key, value in params.items():
+        if value is None and not allow_none:
+            continue
         if type(value) == list:
-            params[key] = (
+            converted_params[key] = (
                 "["
                 + ";".join(
                     [
@@ -64,54 +71,67 @@ def get_api_answer(
                 + "]"
             )
         else:
-            params[key] = json_lib.jsonify_data(value, string_output=True)
+            converted_params[key] = json_lib.jsonify_data(value, string_output=True)
+
+    headers = {}
+    if token is not None or core.api.admin_token is not None:
+        token = core.api.admin_token if token is None else token
+        headers["Authorization"] = f"Bearer {token}"
 
     if not "http" in url:
         url = f"http://localhost:{core.api.port}/{url}"
-    params["requester"] = requester
+    converted_params["requester"] = requester
     if no_log:
-        params["no_log"] = "Y"
+        converted_params["no_log"] = "Y"
     try:
         if method_str.lower() == "post":
-            resp = fct(url=url, data=params)
+            resp = fct(url=url, data=converted_params, headers=headers)
         else:
-            resp = fct(url=url, params=params)
+            resp = fct(url=url, params=converted_params, headers=headers)
     except Exception as ex:
         raise AlphaException(f"Fail to contact {url}", ex=ex)
 
-    if resp.status_code != 200:
-        raise AlphaException(f"Fail to get data from {url=}: {resp.status_code=}")
-
     try:
         answer = resp.json()
     except Exception as ex:
         raise AlphaException(f"Cannot decode answer from {url=}", ex=ex)
+
     if (
         type(answer) == dict
         and answer.get("status", None) is not None
         and answer.get("status_description", None) is not None
     ):
-        return ApiAnswer(**answer)
+        api_answer = ApiAnswer.map_from_dict(answer)
     else:
-        return ApiAnswer(
-            data=answer,
-            token_status=resp.headers.get("x-token-status"),
-            status_code=resp.status_code,
-            error=int(resp.headers.get("x-error")),
-            warning=int(resp.headers.get("x-warning")),
-            status_description=resp.headers.get("x-status-description"),
-            requester=resp.headers.get("x-requester"),
+        api_answer = ApiAnswer.map_from_dict(
+            {
+                "data": answer,
+                "token_status": resp.headers.get("x-token-status"),
+                "status": resp.headers.get("x-status"),
+                "status_code": resp.status_code,
+                "error": int(resp.headers.get("x-error")),
+                "warning": int(resp.headers.get("x-warning")),
+                "status_description": resp.headers.get("x-status-description"),
+                "requester": resp.headers.get("x-requester"),
+            }
         )
 
+    pagination = resp.headers.get("x-pagination", {})
+    if isinstance(pagination, str):
+        pagination_dict = json.loads(pagination.replace("'", '"'))
+        pagination = ApiPagination.map_from_dict(pagination_dict)
+    api_answer.pagination = pagination
+    return api_answer
+
 
 def get_api_data(
     url: str,
     params: dict = {},
-    log: AlphaLogger = None,
+    log: AlphaLogger | None = None,
     method: ApiMethods = ApiMethods.GET,
     data_only: bool = True,
-    requester="EXT",
-) -> dict:
+    requester=None,
+) -> dict | ApiAnswer:
     """Get data from api
 
     Args:
@@ -137,7 +157,7 @@ def get_api_data(
 
 
 def get_routes_infos(
-    log: AlphaLogger = None, categories=None, routes=None, reload_=False
+    log: AlphaLogger | None = None, categories=None, routes=None, reload_=False
 ) -> dict:
     """Get all apis routes with informations.
 
