@@ -1,11 +1,11 @@
-import datetime, os, re, configparser, pickle, json
-from typing import List
-
-CONFIG_FILE_EXTENSION = ".ini"
-
+# MODULES
+import datetime, os, re, configparser, pickle
 from collections import OrderedDict
 
+# LIBS
 from ..libs import converter_lib, json_lib, io_lib, dict_lib
+
+# MODELS
 from ..models.database.structure import AlphaDatabase
 
 import numpy as np
@@ -13,6 +13,7 @@ import numpy as np
 CONSTANTS = {}
 PARAMETERS = {}
 CONFIGS = {}
+CONFIG_FILE_EXTENSION = ".ini"
 
 
 def un_pickle(value):
@@ -25,21 +26,20 @@ def order(D: dict) -> OrderedDict:
     return OrderedDict(sorted(D.items(), key=lambda x: len(x[0]), reverse=True))
 
 
-def get_db_constants(db: AlphaDatabase) -> OrderedDict:
+def get_db_constants(bind: str | None = None) -> OrderedDict:
+    from core import DB
+
     global CONSTANTS
-    from core import core
 
     """Get constants from database
 
     Args:
-        db (AlphaDatabase): [description]
+        bind (str): [description]
 
     Returns:
         OrderedDict: constants in a dict with <name> key and <value> value
     """
-    model = core.get_table(db.name, "constants")
-
-    rows = db.select(model, json=True)
+    rows = DB.select(Constants, json=True)
     values = {x["name"]: x["value"] for x in rows}
 
     now = datetime.datetime.now()
@@ -51,15 +51,15 @@ def get_db_constants(db: AlphaDatabase) -> OrderedDict:
     values["second"] = now.second
 
     order(values)
-    CONSTANT = values
+    CONSTANTS = values
     return CONSTANTS
 
 
-def get_db_parameters(db: AlphaDatabase) -> OrderedDict:
-    global PARAMETERS
-    from core import core
+def get_db_parameters(bind: str | None = None) -> OrderedDict:
+    from core import DB, core
 
     LOG = core.get_logger("config")
+    global PARAMETERS
     """[summary]
 
     Args:
@@ -69,9 +69,9 @@ def get_db_parameters(db: AlphaDatabase) -> OrderedDict:
     Returns:
         [type]: [description]
     """
-    model = core.get_table(db.name, "parameters")
+    model = DB.get_table_model("parameters", bind=bind)
 
-    rows = db.select(model)
+    rows = DB.select(model)
     values = {}
     for row in rows:
         try:
@@ -83,7 +83,9 @@ def get_db_parameters(db: AlphaDatabase) -> OrderedDict:
     return PARAMETERS
 
 
-def get_db_constant(db: AlphaDatabase, name: str, update: bool = False, separator=None):
+def get_db_constant(
+    name: str, update: bool = False, separator=None, bind: str | None = None
+) -> list[str]:
     """[summary]
 
     Args:
@@ -97,13 +99,15 @@ def get_db_constant(db: AlphaDatabase, name: str, update: bool = False, separato
         [type]: [description]
     """
     if update:
-        get_db_constants(db)
+        get_db_constants(bind=bind)
 
     values = None
-    if not is_db_constant(db, name):
-        get_db_constants(db)
-        if is_db_constant(db, name):
+    if not is_db_constant(name, bind=bind):
+        get_db_constants(bind=bind)
+        if is_db_constant(name, bind=bind):
             values = CONSTANTS[name]
+    else:
+        values = CONSTANTS[name]
 
     if separator is not None:
         if values is not None:
@@ -118,34 +122,34 @@ def get_db_constant(db: AlphaDatabase, name: str, update: bool = False, separato
     return values
 
 
-def get_db_parameter(db: AlphaDatabase, name: str, update: bool = False):
+def get_db_parameter(name: str, update: bool = False, bind: str | None = None):
     """[summary]
 
     Args:
-        db ([type]): [description]
         model ([type]): [description]
         core_ ([type]): [description]
         name ([type]): [description]
         update (bool, optional): [description]. Defaults to False.
+        bind (str): [description]
 
     Returns:
         [type]: [description]
     """
     values = None
     if update:
-        get_db_parameters(db)
+        get_db_parameters(bind=bind)
 
-    if not is_db_parameter(db, name):
-        get_db_parameters(db)
-        if is_db_parameter(db, name):
+    if not is_db_parameter(name, bind=bind):
+        get_db_parameters(bind=bind)
+        if is_db_parameter(name, bind=bind):
             values = PARAMETERS[name]
     else:
         values = PARAMETERS[name]
     return values
 
 
-def set_db_constant(db: AlphaDatabase, name: str, value):
-    from core import core
+def set_db_constant(name: str, value, bind: str | None = None):
+    from core import DB
 
     """[summary]
 
@@ -156,12 +160,12 @@ def set_db_constant(db: AlphaDatabase, name: str, value):
         value ([type]): [description]
     """
     CONSTANTS[name] = value
-    model = core.get_table(db.name, "constants")
-    db.insert_or_update(model, values={"name": name, "value": value})
+    model = DB.get_table_model("constants", bind=bind)
+    DB.add_or_update(model, parameters={"name": name, "value": value})
 
 
-def set_db_parameter(db: AlphaDatabase, name: str, value):  # TODO: set core
-    from core import core
+def set_db_parameter(name: str, value, bind: str | None = None):  # TODO: set core
+    from core import DB
 
     """[summary]
 
@@ -172,13 +176,13 @@ def set_db_parameter(db: AlphaDatabase, name: str, value):  # TODO: set core
         value ([type]): [description]
     """
     PARAMETERS[name] = value
-    model = core.get_table(db.name, "parameters")
+    model = DB.get_table_model("parameters", bind=bind)
     # data                = json.dumps(json_lib.jsonify_data(value))
     data = pickle.dumps(value)
-    db.insert_or_update(model, values={"name": name, "value": data})
+    DB.add_or_update(model, parameters={"name": name, "value": data})
 
 
-def is_db_constant(db: AlphaDatabase, name: str):
+def is_db_constant(name: str, bind: str | None = None):
     """[summary]
 
     Args:
@@ -191,7 +195,7 @@ def is_db_constant(db: AlphaDatabase, name: str):
     return name in CONSTANTS
 
 
-def is_db_parameter(db: AlphaDatabase, name: str):
+def is_db_parameter(name: str, bind: str | None = None):
     """[summary]
 
     Args:
@@ -204,7 +208,7 @@ def is_db_parameter(db: AlphaDatabase, name: str):
     return name in PARAMETERS
 
 
-def get_api_build(db: AlphaDatabase, update: bool = False):
+def get_api_build(update: bool = False, bind: str | None = None):
     """[summary]
 
     Args:
@@ -215,13 +219,13 @@ def get_api_build(db: AlphaDatabase, update: bool = False):
     Returns:
         [type]: [description]
     """
-    build = get_db_constant(db, "api_build", update=update)
+    build = get_db_constant("api_build", update=update, bind=bind)
     if build is None:
         build = 0
     return build
 
 
-def upgrade_api_build(db: AlphaDatabase):
+def upgrade_api_build(bind: str | None = None):
     """[summary]
 
     Args:
@@ -229,8 +233,8 @@ def upgrade_api_build(db: AlphaDatabase):
         model ([type]): [description]
         core_ ([type]): [description]
     """
-    build = get_api_build(db)
-    set_db_constant(db, "api_build", int(build) + 1)
+    build = get_api_build()
+    set_db_constant("api_build", int(build) + 1, bind=bind)
 
 
 def get_config_filename(fileName, directory, test=False):
@@ -365,7 +369,7 @@ def convert_config_value(value):
         # match = stringMatch.groups()[0]
         # value = match.replace('\'','')
         value = value[1:-1]
-    elif valueStr == "NONE" or valueStr is None:
+    elif str(value) == "None" or valueStr is None:
         value = None
     elif valueStr == "FALSE":
         value = False
@@ -453,7 +457,7 @@ def write_flask_dashboard_configuration():
         return None
 
 
-def get_configs(config: str = None):
+def get_configs(config: str | None = None):
     from core import DB, LOG
     from ..models.database.main_definitions import Configs
 
@@ -463,16 +467,29 @@ def get_configs(config: str = None):
     for config in configs:
         keys = [x.lower() for x in config.name.split(":")]
         value = config.value
-        type = config.type.upper().strip()
+        config_type = config.type.upper().strip()
 
-        if type == "BOOLEAN":
-            value = True if "T" in value.upper() else False
-        elif type == "STRING":
-            value = str(value)
-        elif type == "INT":
-            value = int(value)
-        elif type == "STRING[]":
-            value = str(value).split(";")
+        try:
+            match config_type:
+                case "BOOLEAN":
+                    value = True if "T" in value.upper() else False
+                case "STRING":
+                    value = str(value)
+                case "STRING[]":
+                    value = str(value).split(";")
+                case "INT":
+                    value = int(value)
+                case "INT[]":
+                    value = [int(x) for x in str(value).split(";")]
+                case "FLOAT":
+                    value = float(value)
+                case "FLOAT[]":
+                    value = [float(x) for x in str(value).split(";")]
+        except:
+            LOG.error(
+                f"Failed to convert config parameter {config.name} to {config_type}"
+            )
+            value = None
 
         dict_keys = dict_lib.get_nested_dict_from_list(keys)
         dict_lib.set_nested(dict_keys, keys, value)
@@ -481,10 +498,10 @@ def get_configs(config: str = None):
     return configs_dict
 
 
-def get_configs_key(keys: List[str], config: str = None, default=None):
+def get_configs_key(keys: str | list[str], config: str | None = None, default=None):
     global CONFIGS
     if type(keys) == str:
-        keys = keys.split(";")
+        keys = keys.split(":")
     if CONFIGS is None or len(CONFIGS) == 0:
         CONFIGS = get_configs(config)
     value = dict_lib.get_nested(CONFIGS, [x.lower() for x in keys])
