@@ -1,4 +1,5 @@
 from datetime import datetime
+
 from core import core
 from ..models.database.main_definitions import Logs
 from ..models.main import AlphaException
@@ -37,7 +38,7 @@ def log_a_message(
     criticality: str,
     message: str,
     level: int,
-    monitor: str = None,
+    monitor: str | None = None,
     log_in_db: bool = False,
 ):
     log = core.get_logger(logger)
