@@ -1,10 +1,14 @@
 from ..models.database.users_definitions import Notification, User
 
+from core import core
+
+db = core.db
+
 
 def add_notifications(
-    api, db, element_type, element_action, id, users=None, all_users=True
+    user, element_type, element_action, id, users=None, all_users=True
 ):
-    user = api.get_logged_user()
+    return
     if all_users:
         users = db.select(
             User, distinct=User.id, columns=[User.id], filters=[User.id != user.id]
@@ -12,7 +16,7 @@ def add_notifications(
 
     notifications = [
         Notification(
-            user=x.id,
+            user=x[0],
             user_from=user.id,
             element_type=element_type,
             element_action=element_action,
