@@ -1,8 +1,16 @@
-from threading import current_thread
-import paramiko, encodings, scp, re, datetime, os
-from typing import Dict, List
+# MODULES
+import scp, re, datetime, os
+import warnings
+from cryptography.utils import CryptographyDeprecationWarning
 
+with warnings.catch_warnings():
+    warnings.filterwarnings("ignore", category=CryptographyDeprecationWarning)
+    import paramiko
+
+# MODELS
 from ..models.main import AlphaFile
+
+# LIBS
 from .string_lib import universal_decode
 from . import io_lib
 
@@ -10,25 +18,56 @@ WINDOWS_LINE_ENDING = "\r\n"
 UNIX_LINE_ENDING = "\n"
 
 
-def standardize_content(content: str):
-    if type(content) != str:
+def standardize_content(content: str) -> str:
+    """
+    Standardize the formatting of a string.
+
+    Replaces occurrences of "\r\n", "\\r\\n", and "\\r\n" with "\n" and replaces occurrences of "\\t" with "\t".
+
+    Args:
+        content: A string to standardize.
+
+    Returns:
+        The standardized string.
+    """
+    if not isinstance(content, str):
         content = content.decode("utf-8")
     return content.replace("\\r\\n", "\n").replace("\\r\n", "\n").replace("\\t", "\t")
 
 
-def process_content(content: str, parameters: Dict[str, object] = None):
+def process_content(content: str, parameters: dict[str, object] | None = None) -> str:
+    """
+    Process the content of a string.
+
+    If the string starts with "file:", the rest of the string is interpreted as a file path and the contents of the file
+    are read and returned. If `parameters` is not None, the function replaces occurrences of "{{key}}" in the content
+    string with the corresponding value in `parameters`.
+
+    Args:
+        content: A string to process.
+        parameters: A dictionary of key-value pairs to replace in the content string.
+
+    Returns:
+        The processed content string.
+    """
     if content.startswith("file:"):
         path = content.replace("file:", "")
         if not os.path.exists(path):
-            path = os.getcwd() + os.sep + path
+            path = os.path.join(os.getcwd(), path)
         with open(path, "r") as f:
             content = f.read()
     if parameters is not None:
         for key, value in parameters.items():
-            content = content.replace("{{%s}}" % key, value)
+            content = content.replace(f"{key}", str(value))
     return content
 
 
+def split_lines(content: str):
+    return content.split(
+        "\r\r\n" if "\r\r\n" in content else ("\r\n" if "\r\n" in content else "\n")
+    )
+
+
 class AlphaSsh:
     host = None
     user = None
@@ -91,56 +130,74 @@ class AlphaSsh:
         while not ssh_stdout.channel.exit_status_ready():
             pass
 
-    def list_files(self, directory: str) -> List[AlphaFile]:
+    def list_files(self, directory: str) -> list[AlphaFile]:
         """[summary]
 
         Args:
             directory (str): [description]
 
         Returns:
-            List[AlphaFile]: [description]
+            list[AlphaFile]: [description]
         """
         output = self.execute_cmd(f"ls -l {directory}")
         files = io_lib.get_list_file(output)
         return files
 
     def list_files_names(
-        self, directory: str, pattern: str = None, hidden: bool = False
-    ) -> List[str]:
-        cmd = "ls -l -f %s" % directory
-        output = self.execute_cmd(cmd)
-        lines = str(output).split()
+        self, directory: str, pattern: str | None = None, hidden: bool = False
+    ) -> list[str]:
+        cmd = "ls -lt %s" % directory
+        cmd = f"find {directory} -maxdepth 1 -type f"
         if pattern is not None:
-            """filtered = []
-            for line in lines:
-                matchs = re.findall(pattern,line)
-                if matchs:
-                    filtered.append(line)
-            lines = filtered"""
-            lines = [x for x in lines if len(re.findall(pattern, x)) != 0]
+            cmd += f' -iname "{pattern}"'
+
+        output = self.execute_cmd(cmd)
+        lines = split_lines(str(output))
+        # if pattern is not None:
+        """filtered = []
+        for line in lines:
+            matchs = re.findall(pattern,line)
+            if matchs:
+                filtered.append(line)
+        lines = filtered"""
+        # lines = [x for x in lines if len(re.findall(pattern, x)) != 0]
+
+        files_names = []
+        for line in lines:
+            if line.strip() == "":
+                continue
+            elements = line.split()
+
+            # Récupérer le dernier élément, qui est le chemin complet du fichier.
+            full_file_path = elements[-1]
+
+            # Utiliser les opérations de découpage de chemin pour obtenir le nom de fichier.
+            file_name = full_file_path.split("/")[-1]
+            files_names.append(file_name)
+
         if hidden:
+            return [x for x in files_names if x.replace(".", "") != ""]
+        else:
             return [
                 x
-                for x in lines
-                if x.replace(".", "") != "" and (not "." in x or x.startswith("."))
+                for x in files_names
+                if x.replace(".", "") != "" and not x.startswith(".")
             ]
-        else:
-            return [x for x in lines if x.replace(".", "") != "" and not "." in x]
 
-    def list_directories(self, directory: str) -> List[AlphaFile]:
+    def list_directories(self, directory: str) -> list[AlphaFile]:
         """[summary]
 
         Args:
             directory (str): [description]
 
         Returns:
-            List[AlphaFile]: [description]
+            list[AlphaFile]: [description]
         """
         output = self.execute_cmd("ls -l %s" % directory)
         directories = io_lib.get_list_file(output)
         return directories
 
-    def list_directories_names(self, directory: str, hidden: bool = False) -> List[str]:
+    def list_directories_names(self, directory: str, hidden: bool = False) -> list[str]:
         output = self.execute_cmd("ls -l -f %s" % directory)
         lines = str(output).split()
         if hidden:
@@ -162,10 +219,22 @@ class AlphaSsh:
         output = self.execute_cmd("test -f %s && echo 'y'" % filename)
         return "y" in output
 
-    def is_directory(self, path: str, group: str = None, user: str = None, mode=None):
+    def is_directory(
+        self,
+        path: str,
+        group: str | None = None,
+        user: str | None = None,
+        mode=None,
+    ):
         return self.is_dir(path=path, group=group, user=user, mode=mode)
 
-    def is_dir(self, path: str, group: str = None, user: str = None, mode=None):
+    def is_dir(
+        self,
+        path: str,
+        group: str | None = None,
+        user: str | None = None,
+        mode=None,
+    ):
         output = self.execute_cmd("test -d %s && echo 'y'" % path)
         if group is not None and not self.is_group(group, path):
             return False
@@ -178,8 +247,8 @@ class AlphaSsh:
     def make_directory(
         self,
         path: str,
-        group: str = None,
-        user: str = None,
+        group: str | None = None,
+        user: str | None = None,
         mode=None,
         ensure_path: bool = True,
     ):
@@ -261,7 +330,7 @@ class AlphaSsh:
     def restart_service(self, service: str):
         self.execute_cmd("sudo systemctl restart %s" % service)
 
-    def is_equal_to_file(self, content: str, path: str, mode: int = None):
+    def is_equal_to_file(self, content: str, path: str, mode: int | None = None):
         if not self.is_file(path):
             return False
 
@@ -309,7 +378,7 @@ class AlphaSsh:
         content: str,
         path: str,
         ensure_path: bool = True,
-        parameters: Dict[str, object] = None,
+        parameters: dict[str, object] = None,
     ) -> bool:
         if ensure_path:
             self.make_directory(os.path.dirname(path))
@@ -353,7 +422,7 @@ class AlphaSsh:
         output = self.execute_cmd(cmd)
         return len(output.replace("\\n", "").replace("\\r", "").strip()) != 0
 
-    def is_found(self, cmd: str, greps: List[str] = []):
+    def is_found(self, cmd: str, greps: list[str] = []):
         if type(greps) == str:
             greps = [greps]
         cmd = cmd + " | " + " | ".join(['grep "%s"' % x for x in greps])
@@ -364,7 +433,7 @@ class AlphaSsh:
             return False
         return len(output) != 0
 
-    def get_pid(self, greps: List[str] = []):
+    def get_pid(self, greps: list[str] = []):
         if type(greps) == str:
             greps = [greps]
         cmd = "ps aux -P | " + " | ".join(['grep "%s"' % x for x in greps])
@@ -418,7 +487,7 @@ class AlphaSsh:
         valid = not ("No module named " in output and module in output)
         return valid
 
-    def install_python_module(self, module: str, version: str = None):
+    def install_python_module(self, module: str, version: str | None = None):
         cmd = "which pip"
         output = self.execute_cmd(cmd)
         cmd = f"yes | pip install {module}{version}"
@@ -428,9 +497,9 @@ class AlphaSsh:
     def add_user(
         self,
         user: str,
-        description: str = None,
-        group: str = None,
-        password: str = None,
+        description: str | None = None,
+        group: str | None = None,
+        password: str | None = None,
     ):
         options = ""
         if description is not None:
@@ -445,7 +514,7 @@ class AlphaSsh:
             cmd = 'echo "%s" | sudo passwd --stdin %s' % (password, user)
             self.execute_cmd(cmd)
 
-    def execute_cmd(self, cmd, decode=True, lines=False, timeout: int = 600):
+    def execute_cmd(self, cmd, decode=True, lines=False, timeout: int = 1000):
         inputs, output, err = "", "", ""
         if self.log:
             self.log.info(f"EXEC: {cmd}")
