@@ -1,14 +1,13 @@
-import os, imp, sys, inspect
+# MODULES
+from .. import os, defaultdict, timer
 
-from . import io_lib
-from ..models.tests import TestGroup, AlphaTest, test, TestCategories
-import importlib
-from inspect import getmembers, isfunction, isclass
-from .py_lib import reload_modules
+# LIBS
+from . import io_lib, py_lib
 
-from ..models.logger import AlphaLogger
-from typing import List
+# MODELS
+from ..models.tests import TestGroup, AlphaTest, TestCategories, Levels
 
+# CORE
 from core import core
 
 LOG = core.get_logger("tests")
@@ -16,33 +15,14 @@ LOG = core.get_logger("tests")
 CATEGORIES = {}
 
 
-def get_tests_auto(
-    tests_modules: List[str],
-    name: str = None,
-    names: List[str] = [],
-    group: str = None,
-    groups: List[str] = [],
-    category: str = None,
-    categories: List[str] = [],
-    file_path: str = None,
-    run: bool = False,
-    resume: bool = False,
-    coverage: str = None,
-) -> TestCategories:
-    """Get the TestCategories class, containings all required tests.
-
-    Args:
-        tests_modules (List[str]): list of test modules path
-        tests_modules (List[str]): list of test modules path
-        name (str, optional): the name of the test to select. Defaults to None.
-        group (str, optional): the name of the group to select. Defaults to None.
-        category (str, optional): the name of the category to select. Defaults to None.
-
-    Returns:
-        TestCategories: [description]
-    """
-    global CATEGORIES
-
+def __group_parameters(
+    name: str | None = None,
+    names: list[str] | None = None,
+    group: str | None = None,
+    groups: list[str] | None = None,
+    category: str | None = None,
+    categories: list[str] | None = None,
+):
     categories = categories if categories is not None else []
     groups = groups if groups is not None else []
     names = names if names is not None else []
@@ -56,44 +36,35 @@ def get_tests_auto(
     categories = [x.strip() for x in categories]
     groups = [x.strip() for x in groups]
     names = [x.strip() for x in names]
+    return categories, groups, names
 
-    if len(categories) == 0 and len(groups) == 0 and len(names) == 0:
-        LOG.info("Requested execution of all tests")
-    else:
-        LOG.info(
-            f"Requested execution of tests:\n - categories: {', '.join(categories)}\n - groups: {', '.join(groups)}\n - names: {', '.join(names)}"
-        )
+
+def process_test_categories(
+    categories,
+    groups: list[str] = [],
+    names: list[str] = [],
+    levels: list[Levels] = [],
+    run: bool = False,
+    coverage: str | None = None,
+    load_from_db: bool = True,
+    stop: bool = True,
+    failed_only: bool = False,
+):
+    subclasses = AlphaTest.__subclasses__()
+    subclasses_by_module = defaultdict(list)
+    for subclass in subclasses:
+        module_name = subclass.__module__.split(".")[0]
+        subclasses_by_module[module_name].append(subclass)
 
     test_categories = TestCategories()
-
-    for tests_module in tests_modules:
-        try:
-            print(f"Import module {tests_module}")
-            module = importlib.import_module(tests_module)
-        except Exception as ex:
-            LOG.error(f"Cannot load test module <{tests_module}>", ex=ex)
-            continue
-
-        class_list = []
-        for o in getmembers(module):
-            is_class = isclass(o[1])
-            if not is_class:
-                continue
-            is_test = (
-                issubclass(o[1], AlphaTest)
-                and (not hasattr(o[1], "_test") or o[1]._test)
-                and not o[1] == AlphaTest
-            )
-            if not is_test:
-                continue
-            class_list.append(o)
-
+    for category, class_list in subclasses_by_module.items():
+        groups_str = ", ".join([x.__name__ for x in class_list])
         LOG.info(
-            f'Loaded test module <{tests_module}> with test groups: {", ".join([x[0] for x in class_list])}'
+            f"Loaded test module <{category}> with test groups: {groups_str[:100]} ..."
         )
 
         for el in class_list:
-            test_group = TestGroup(el[0], el[1], coverage=coverage is not None)
+            test_group = TestGroup(el, coverage=coverage is not None)
 
             if len(categories) != 0 and test_group.category not in categories:
                 continue
@@ -103,19 +74,74 @@ def get_tests_auto(
             if LOG is not None:
                 LOG.debug(f"Found function group <{test_group.name}>")
 
-            test_group.get_from_database()
-            if run and len(names) == 0:
-                test_group.test_all()
-            elif run:
-                test_group.test_all(names)
-
             test_categories.add_test_group(test_group)
 
+    if load_from_db or failed_only:
+        test_categories.get_from_database()
+    if run:
+        test_categories.test_all(names, levels, stop=stop, failed_only=failed_only)
+
+    return test_categories
+
+
+def get_tests_auto(
+    name: str | None = None,
+    names: list[str] | None = None,
+    group: str | None = None,
+    groups: list[str] | None = None,
+    category: str | None = None,
+    categories: list[str] | None = None,
+    file_path: str | None = None,
+    run: bool = False,
+    load_from_db: bool = False,
+    report: str | None = None,
+    coverage: str | None = None,
+    levels: list[Levels] | None = None,
+    stop: bool = False,
+    failed_only: bool = False,
+) -> TestCategories:
+    """Get the TestCategories class, containings all required tests.
+
+    Args:
+        tests_modules (list[str]): list of test modules path
+        tests_modules (list[str]): list of test modules path
+        name (str, optional): the name of the test to select. Defaults to None.
+        group (str, optional): the name of the group to select. Defaults to None.
+        category (str, optional): the name of the category to select. Defaults to None.
+
+    Returns:
+        TestCategories: [description]
+    """
+    global CATEGORIES
+
+    categories, groups, names = __group_parameters(
+        name, names, group, groups, category, categories
+    )
+
+    if run and len(categories) == 0 and len(groups) == 0 and len(names) == 0:
+        LOG.info("Requested execution of all tests")
+    elif run:
+        LOG.info(
+            f"Requested execution of tests:\n - categories: {', '.join(categories)}\n - groups: {', '.join(groups)}\n - names: {', '.join(names)}"
+        )
+
+    test_categories = process_test_categories(
+        categories,
+        groups,
+        names,
+        levels,
+        run,
+        coverage,
+        load_from_db=load_from_db,
+        stop=stop,
+        failed_only=failed_only,
+    )
+
     if file_path is not None:
         test_categories.to_junit(file_path)
 
-    if resume:
-        test_categories.resume()
+    if report is not None:
+        test_categories.report(report)
 
     if coverage is not None:
         coverages = {}
@@ -155,9 +181,9 @@ def operate_all_tests_auto(
     import_path=None,
 ):
     if refresh:
-        reload_modules(os.getcwd())
+        py_lib.reload_modules(os.getcwd())
 
-    test_categories = get_tests_auto(directory, group=group, import_path=import_path)
+    test_categories = get_tests_auto(group=group, import_path=import_path)
 
     if action == "execute":
         tests_groups.test_all()
