@@ -1,6 +1,5 @@
 from alphaz.models.database.users_definitions import Application
 from core import core
-from typing import List
 
 
 DB = core.db
@@ -13,7 +12,7 @@ def get_applications(
     page_size: int,
     order_by: str,
     direction: str,
-    columns: List[str] = None,
+    columns: list[str] = None,
 ):
     return DB.select(
         Application,
@@ -26,15 +25,17 @@ def get_applications(
         columns=columns,
     )
 
-def get_applications_names(name:str):
+
+def get_applications_names(name: str):
     return DB.select(
         Application,
-        optional_filters={Application.name:{'like':name}},
+        optional_filters={Application.name: {"like": name}},
         unique=Application.name,
         distinct=Application.name,
-        order_by=Application.name.asc()
+        order_by=Application.name.asc(),
     )
 
+
 def get_applications_names_and_id():
     return DB.select(Application, columns=[Application.name, Application.id])
 
