@@ -1,5 +1,3 @@
-from typing import List
-
 from sqlalchemy import distinct, or_
 from sqlalchemy.sql.expression import text
 from alphaz.models.database.users_definitions import (
@@ -25,7 +23,7 @@ def get_roles(
     page_size: int,
     order_by: str,
     direction: str,
-) -> List[AlphaRole]:
+) -> list[AlphaRole]:
     query = (
         DB.session.query(Role)
         .distinct(Role.name)
@@ -106,7 +104,10 @@ def add_role_permission(perm_key: str, role_name: str):
 def delete_user_role(user_id: int, role_name: str):
     return DB.delete(
         UserRole,
-        filters=[UserRole.user_id == user_id, UserRole.role_name == role_name,],
+        filters=[
+            UserRole.user_id == user_id,
+            UserRole.role_name == role_name,
+        ],
     )
 
 
