@@ -1,6 +1,6 @@
 from datetime import datetime
-import json
-from typing import List
+import ujson
+
 
 from sqlalchemy import or_, and_, outerjoin
 from alphaz.models.database.users_definitions import (
@@ -29,7 +29,7 @@ def get_users(
     direction: str,
     page_index: int = 0,
     page_size: int = 200,
-) -> List[AlphaUser]:
+) -> list[AlphaUser]:
     query = (
         DB.session.query(User)
         .distinct(User.username)
@@ -85,6 +85,6 @@ def add_user(uid: str):
         "username": res["uid"],
         "mail": res["mail"] if "mail" in res else "-",
         "date_registred": datetime.now(),
-        "infos": json.dumps(added_infos),
+        "infos": ujson.dumps(added_infos),
     }
     return DB.add(User(**user))
