@@ -1,18 +1,105 @@
-import json, datetime, dataclasses, re, typing, sqlalchemy, xmltodict
+# MODULES
+import ujson, datetime, re, sqlalchemy, xmltodict
 
-from sqlalchemy.orm.base import object_mapper
-from sqlalchemy.orm.exc import UnmappedInstanceError
 from sqlalchemy.ext.declarative import DeclarativeMeta
 from sqlalchemy import inspect as inspect_sqlalchemy
 
 from flask import request
 from collections.abc import Callable
 
+# MAIN
 from ..main import AlphaException
+from ..main._base import is_dataclass
+from ..main._enum import MappingMode, ParameterMode
 
-from ...libs import date_lib, json_lib, py_lib, flask_lib
+# LIBS
+from ...libs import date_lib, json_lib, py_lib, string_lib
 
-from enum import Enum
+
+# TODO: remove form
+def convert_parameter_value(
+    name: str,
+    value,
+    ptype,
+    form,
+    mode=None,
+    map_none: bool = False,
+    required: bool = False,
+    mapping_mode: MappingMode | None = None,
+):
+    str_value = str(value).lower()
+    if py_lib.is_subtype(ptype, bool):
+        if str_value in ["y", "true", "t", "1"]:
+            value = True
+        elif str_value in ["n", "false", "f", "0"]:
+            value = False
+        elif not required and value is None:
+            value = None
+        else:
+            raise AlphaException(
+                f"Wrong value <{value}> for parameter <{name}> of type <bool>",
+            )
+        value = value
+    elif py_lib.is_subtype(ptype, int):
+        try:
+            value = int(value) if value is not None else None
+        except:
+            raise AlphaException(
+                f"Wrong value <{value}> for parameter <{name}> of type <int>"
+            )
+    elif py_lib.is_subtype(ptype, float):
+        try:
+            value = float(value)
+        except:
+            raise AlphaException(
+                f"Wrong value <{value}> for parameter <{name}> of type <float>"
+            )
+    elif ptype == datetime.datetime:
+        value = date_lib.str_to_datetime(value)
+    elif py_lib.is_subtype(ptype, dict):
+        value = json_lib.load_json(value)
+    elif str_value in ["null()"]:
+        value = sqlalchemy.null()
+    elif str_value in [
+        "null",
+        "none",
+        "undefined",
+    ]:
+        value = None
+    elif is_dataclass(ptype):
+        if hasattr(ptype, "map_from_json"):
+            value = ptype.map_from_json(value, mapping_mode=mapping_mode)
+        else:
+            try:
+                P = ujson.loads(value)
+            except Exception as ex:
+                raise ex
+            value = ptype(**P)
+    elif isinstance(ptype, DeclarativeMeta):  # DB MODEL
+        if value is None:
+            parameters = {x: y for x, y in form.items() if hasattr(ptype, x)}
+        else:
+            parameters = value if type(value) == dict else json_lib.load_json(value)
+            parameters = {
+                x: y
+                for x, y in parameters.items()
+                if hasattr(ptype, x) and (map_none or y is not None)
+            }  # TODO: enhance
+        value = dict_to_model(ptype, parameters)
+    elif hasattr(ptype, "metadata") and not hasattr(
+        value, "metadata"
+    ):  # classic dataclass
+        r = ujson.loads(value)
+        value = ptype(**r)
+    elif py_lib.is_subtype(ptype, str):
+        if mode in [
+            ParameterMode.LIKE,
+            ParameterMode.IN_LIKE,
+            ParameterMode.START_LIKE,
+            ParameterMode.END_LIKE,
+        ]:
+            value = set_value_like_mode(value, mode)
+    return value
 
 
 def dict_to_model(model, parameters):
@@ -37,7 +124,7 @@ def dict_to_model(model, parameters):
     for relationship, relationship_property in inspected.relationships.items():
         if relationship in parameters:
             relationship_table_name = relationship_property.target.key
-            relationship_model = core.get_table_model(relationship_table_name)
+            relationship_model = core.db.get_table_model(relationship_table_name)
             parameters[relationship] = dict_to_model(
                 relationship_model, parameters[relationship]
             )
@@ -71,20 +158,6 @@ def set_value_like_mode(value, mode):
     return value
 
 
-class ParameterMode(Enum):
-    NONE = 0
-    LIKE = 1
-    IN_LIKE = 2
-    START_LIKE = 3
-    END_LIKE = 4
-
-    def __str__(self):
-        return str(self.value)
-
-    def to_json(self):
-        return str(self.value)
-
-
 class Parameter:
     _value = None
     no_log: bool = False
@@ -102,11 +175,13 @@ class Parameter:
         private: bool = False,
         mode: ParameterMode = ParameterMode.NONE,
         override: bool = False,
-        function: Callable = None,
-        automap: bool = False,
+        function: Callable | None = None,
+        mapping_mode: MappingMode | str | None = None,
         map_none: bool = False,
-        max_size: int = None,
-        min_size: int = None,
+        max_size: int | None = None,
+        min_size: int | None = None,
+        max_value: int | None = None,
+        min_value: int | None = None,
         separators: list = [",", ";"],
     ):
         """[summary]
@@ -135,14 +210,16 @@ class Parameter:
         self.mode = mode
         self.override = override
         self.is_value = False
-        self.automap = automap
+        self.mapping_mode = mapping_mode
         self.max_size = max_size
         self.min_size = min_size
         self.map_none = map_none
+        self.max_value = max_value
+        self.min_value = min_value
 
     @property
     def value(self):
-        return self._value if self._value is not None else self.default
+        return self._value if self._value is not None or self.is_value else self.default
 
     def __check_options(self, value):
         if (
@@ -159,17 +236,7 @@ class Parameter:
                 },
             )
 
-    def set_value(self, method: str, dataDict, api_json, form, args):
-        """Set parameter value
-
-        Raises:
-            AlphaException: [description]
-        """
-        # dataDict: for list GET
-        self._value = self.default
-        self.name = self.name.strip()
-        dataDict = {x.strip(): y for x, y in dataDict.items()}
-
+    def _get_value_from_request(self, dataDict, api_json, form, args):
         if "<xml>" in str(request.data):
             try:
                 data = xmltodict.parse(request.data)
@@ -187,7 +254,7 @@ class Parameter:
         elif (
             self.name in args
             and self.name in dataDict
-            and (self.ptype == list or py_lib.is_subtype(self.ptype, typing.List))
+            and (self.ptype == list or py_lib.is_subtype(self.ptype, list))
             and (len(dataDict[self.name]) != 1)
         ):
             self._value = dataDict[self.name]
@@ -199,6 +266,37 @@ class Parameter:
             self._value = dataDict[self.name]
             self.is_value = True
 
+    def _check_min_max_value(self):
+        if (
+            self.max_value is not None
+            and self._value is not None
+            and type(self._value) in [int, float]
+            and self._value > self.max_value
+        ):
+            raise AlphaException(
+                "api_parameter_value_above_max",
+                parameters={
+                    "parameter": self.name,
+                    "value": self._value,
+                    "max_value": self.max_value,
+                },
+            )
+        if (
+            self.min_value is not None
+            and self._value is not None
+            and type(self._value) in [int, float]
+            and self._value < self.min_value
+        ):
+            raise AlphaException(
+                "api_parameter_value_below_min",
+                parameters={
+                    "parameter": self.name,
+                    "value": self._value,
+                    "min_value": self.min_value,
+                },
+            )
+
+    def _check_sizes(self):
         # check size
         if (
             self.max_size is not None
@@ -220,7 +318,7 @@ class Parameter:
             and len(self._value) < self.min_size
         ):
             raise AlphaException(
-                "api_parameter_length_too_long",
+                "api_parameter_length_too_short",
                 parameters={
                     "parameter": self.name,
                     "size": len(self.value),
@@ -229,146 +327,67 @@ class Parameter:
                 },
             )
 
-        # List
-        if self.ptype == list or py_lib.is_subtype(self.ptype, typing.List):
-            is_empty = self._value is None or str(self._value).strip() == ""
-            is_empty_value = all(not x in str(self._value) for x in [";", ",", "["])
-
-            if is_empty:
-                self._value = self.default
-            elif is_empty_value:
-                self._value = (
-                    dataDict[self.name] if self.name in dataDict else self.default
-                )
-
-            sub_type = py_lib.get_subtype(self.ptype)
-            if dataclasses.is_dataclass(sub_type):
-                if self._value is not None:
-                    values = re.findall(r"{[^{]*}", str(self._value))
-                    self._value = []
-                    for val in values:
-                        if hasattr(sub_type, "map_from_json"):
-                            val = sub_type.map_from_json(val, automap=self.automap)
-                        else:
-                            P = json.loads(val)
-                            val = sub_type(**P)
-                        self._value.append(val)
-                else:
-                    self._value = []
-
-        if self._value is None and form is not None and self.name in form:
-            self._value = form[self.name]
+    def _check_none(self, form) -> bool:
         if self._value is None and form is not None and self.name in form:
             self._value = form[self.name]
 
-        if isinstance(self.ptype, DeclarativeMeta):
-            if self._value is None:
-                parameters = {x: y for x, y in form.items() if hasattr(self.ptype, x)}
-            else:
-                parameters = (
-                    self._value
-                    if type(self._value) == dict
-                    else json_lib.load_json(self._value)
-                )
-                parameters = {
-                    x: y
-                    for x, y in parameters.items()
-                    if hasattr(self.ptype, x) and (self.map_none or y is not None)
-                }  # TODO: enhance
-            self._value = dict_to_model(self.ptype, parameters)
-        elif dataclasses.is_dataclass(self.ptype):
-            if hasattr(self.ptype, "map_from_json"):
-                self._value = self.ptype.map_from_json(
-                    self._value, automap=self.automap
-                )
-            else:
-                P = json.loads(self._value)
-                self._value = self.ptype(**P)
-
-        if self.ptype == dict:
-            self._value = json_lib.load_json(self._value)
-
-        if self.required and (self._value is None or str(self._value) == "undefined"):
-            raise AlphaException(f"Missing parameter {self.name}")
-
+        if self._value is not None and str(self._value).lower() in [
+            "null",
+            "none",
+            "undefined",
+        ]:
+            self._value = self.none_value if self.is_value else None
+            self.__check_options(self._value)
+            return False
         if self._value is None:
             self.__check_options(self._value)
-            return
+            self._value = self.default
+        if self._value is None and self.required:
+            raise AlphaException(f"Parameter {self.name} is required")
+        return self._value is not None
 
-        if str(self._value).lower() in ["null", "none", "undefined"]:
-            self._value = self.none_value if self.is_value else None
-        if str(self._value).lower() in ["null()"]:
-            self._value = sqlalchemy.null()
-        if self.empty_value is not None and str(self._value).lower() == "":
-            self._value = self.empty_value
+    def _process_list_value(self, dataDict, form) -> bool:
+        if not py_lib.is_subtype(self.ptype, list):
+            return False
+        is_empty = self._value is None or str(self._value) == ""
+        if is_empty:
+            self._value = []
+            return True
 
-        if self.ptype == str and (
-            self.mode
-            in [
-                ParameterMode.LIKE,
-                ParameterMode.IN_LIKE,
-                ParameterMode.START_LIKE,
-                ParameterMode.END_LIKE,
-            ]
-        ):
-            self._value = set_value_like_mode(self._value, self.mode)
+        is_not_list_format = all(not x in str(self._value) for x in [";", ",", "["])
 
-        if self.ptype == bool:
-            str_value = str(self._value).lower()
-            self.__check_options(str(self._value))
+        if is_not_list_format:
+            dataDict = {x.strip(): y for x, y in dataDict.items()}
+            self._value = (
+                dataDict[self.name] if self.name in dataDict else [self._value]
+            )
 
-            if str_value in ["y", "true", "t", "1"]:
-                value = True
-            elif str_value in ["n", "false", "f", "0"]:
-                value = False
-            else:
-                raise AlphaException(
-                    "api_wrong_parameter_value",
-                    parameters={
-                        "parameter": self.name,
-                        "type": "bool",
-                        "value": self._value,
-                    },
+        if self._value is None:
+            return True
+
+        sub_type = py_lib.get_subtype(self.ptype)
+        if is_dataclass(sub_type):
+            values = re.findall(r"{[^{]*}", str(self._value))
+            self._value = []
+            for val in values:
+                if hasattr(sub_type, "map_from_json"):
+                    val = sub_type.map_from_json(val, mapping_mode=self.mapping_mode)
+                else:
+                    P = ujson.loads(val)
+                    val = sub_type(**P)
+                self._value.append(val)
+        else:
+            if not py_lib.is_subtype(self._value, list):
+                self._value = string_lib.str_to_str_list(
+                    str(self._value), separators=[",", ";"]
                 )
-            self._value = value
 
-        if self.ptype == list or py_lib.is_subtype(self.ptype, typing.List):
-            if type(self._value) == str and str(self._value).strip() == "":
-                self._value = []
-            elif type(self._value) == str:
-                try:
-                    if (
-                        len(str(self._value)) != 0
-                        and str(self._value)[0] == "["
-                        and str(self._value)[-1] == "]"
-                    ):
-                        self._value = (
-                            str(self._value)[1:-1].split(";")
-                            if ";" in str(self._value)
-                            else str(self._value)[1:-1].split(",")
-                        )
-                    elif ";" in str(self._value) or "," in str(self._value):
-                        self._value = (
-                            str(self._value).split(";")
-                            if ";" in str(self._value)
-                            else str(self._value).split(",")
-                        )
-                    else:
-                        self._value = [self._value]
-                except:
-                    raise AlphaException(
-                        "api_wrong_parameter_value",
-                        parameters={
-                            "parameter": self.name,
-                            "type": "list",
-                            "value": self._value,
-                        },
-                    )
-
-            if py_lib.is_subtype(self.ptype, typing.List[int]):
-                self._value = [int(x) for x in self._value]
-            if py_lib.is_subtype(self.ptype, typing.List[float]):
-                self._value = [float(x) for x in self._value]
+            self._value = [
+                convert_parameter_value(
+                    self.name, x, py_lib.get_subtype(self.ptype), form
+                )
+                for x in self._value
+            ]
 
             for val in self._value:
                 self.__check_options(val)
@@ -380,45 +399,43 @@ class Parameter:
                 ParameterMode.END_LIKE,
             ]:
                 self._value = [set_value_like_mode(x, self.mode) for x in self._value]
+        return True
 
-        if self.ptype == int:
-            try:
-                self._value = int(self._value) if self._value is not None else None
-            except:
-                raise AlphaException(
-                    "api_wrong_parameter_value",
-                    parameters={
-                        "parameter": self.name,
-                        "type": "int",
-                        "value": self._value,
-                    },
-                )
-            self.__check_options(self._value)
+    def set_value(self, dataDict: dict, api_json, form, args):
+        """Set parameter value
 
-        if self.ptype == float:
-            try:
-                self._value = float(self._value)
-            except:
-                raise AlphaException(
-                    "api_wrong_parameter_value",
-                    parameters={
-                        "parameter": self.name,
-                        "type": "float",
-                        "value": self._value,
-                    },
-                )
-            self.__check_options(self._value)
+        Raises:
+            AlphaException: [description]
+        """
+        dataDict = {x.strip(): y for x, y in dataDict.items()}
 
-        if self.ptype == datetime.datetime:
-            self.__check_options(self._value)
-            self._value = date_lib.str_to_datetime(self._value)
+        self._value = self.default
+        self.name = self.name.strip()
 
-        if hasattr(self.ptype, "metadata") and not hasattr(self._value, "metadata"):
-            r = json.loads(self._value)
-            self._value = self.ptype(**r)
+        self._get_value_from_request(dataDict, api_json, form, args)
+        self._check_sizes()
+        self._check_min_max_value()
 
-        if self.ptype == str:
-            self.__check_options(str(self._value))
+        if not self._check_none(form):
+            return
+
+        # list
+        if not self._process_list_value(dataDict, form):
+            if self.empty_value is not None and str(self._value).lower() == "":
+                self._value = self.empty_value
+
+            self._value = convert_parameter_value(
+                self.name,
+                self._value,
+                self.ptype,
+                form,
+                mode=self.mode,
+                map_none=self.map_none,
+                required=self.required,
+                mapping_mode=self.mapping_mode,
+            )
+
+        self.__check_options(self._value)
 
         if self.function is not None:
             self._value = self.function(self._value)
