@@ -1,16 +1,15 @@
 from flask import request, json
-from typing import *
 
 from ...libs.json_lib import load_json
 
 
 class AlphaRequest:
     @staticmethod
-    def get_gets() -> Dict[str, object]:
+    def get_gets() -> dict[str, object]:
         """returns GET value as a dict.
 
         Returns:
-            Dict[str, object]: [description]
+            dict[str, object]: [description]
         """
         return {x: y for x, y in request.args.items()}
 
@@ -31,16 +30,23 @@ class AlphaRequest:
         return data
 
     @staticmethod
-    def get_uuid():
+    def get_uuid() -> str | None:
+        request_uuid = None
         try:
             data = AlphaRequest.get_json()
             request_uuid = (
-                request.full_path
+                request.method
+                + " - "
+                + request.full_path
                 + "&"
                 + "&".join("%s=%s" % (x, y) for x, y in data.items())
             )
         except:
-            raw_data = request.data
+            try:
+                raw_data = request.data
+            except:
+                return None
+
             if isinstance(raw_data, bytes):
                 if b"\x01\x8d" in raw_data:
                     raw_data = raw_data.replace(b"\x01\x8d", b"\n")
@@ -49,15 +55,20 @@ class AlphaRequest:
         return request_uuid
 
     @staticmethod
-    def get_token():
+    def get_token() -> str | None:
         token = None
         # Get token from authorization bearer
-        auth = request.headers.get("Authorization", None)
+        try:
+            auth = request.headers.get("Authorization", None)
+        except:
+            return None
         if auth is not None:
             if "bearer" in auth.lower():
                 parts = auth.split()
                 if len(parts) > 1:
                     token = parts[1]
+            else:
+                token = auth
 
         # Token from get have priority if present
         token_from_get = request.args.get("token", None)
