@@ -1,14 +1,10 @@
-import datetime, os, copy, json
+import datetime, os, copy, ujson, json
 import time
 import traceback
-from typing import List, Dict, Tuple
-import typing
 
 from flask import (
-    Flask,
     jsonify,
     request,
-    Response,
     make_response,
     render_template,
     send_from_directory,
@@ -19,15 +15,17 @@ from ...libs.time_lib import timer
 from ...libs import json_lib, converter_lib, io_lib, py_lib
 from ...models.main import AlphaException
 from ...models.api import ApiAnswer, AlphaRequest
+from ...models.json._converters import AlphaJSONEncoder
 
 from ._parameter import Parameter
 from ..main._exception import get_message_from_name
 
 SEPARATOR = "__"
+DEBUG = False
 
 
 def get_columns_values_output(
-    objects: list, columns: list = None, header: bool = False
+    objects: list, columns: list | None = None, header: bool = False
 ) -> dict:
     """Generate columns names output columns names + list of list (only the values) + nb of values if not header
     ex : { "columns":["col1","col2",...], "values": ["value1","value2",..], "values_nb":12}
@@ -98,7 +96,7 @@ class Route:
         self,
         uuid: str,
         route: str,
-        parameters: List[Parameter],
+        parameters: list[Parameter],
         request_state,
         api,
         cache: bool = False,
@@ -149,14 +147,18 @@ class Route:
         self.log = api.log
         self.method = request_state.method
 
+        self.page: int = 0
+        self.total, self.total_pages, self.per_page = None, None, None
+        self._is_pagination: bool | None = None
+
         self.init_return()
 
-        self.parameters: Dict[Parameter] = {y.name: copy.copy(y) for y in parameters}
+        self.parameters: dict[str, Parameter] = {
+            y.name: copy.copy(y) for y in parameters
+        }
         for parameter in self.parameters.values():
             try:
-                parameter.set_value(
-                    self.method, self.dict, self.json, self.form, self.args
-                )
+                parameter.set_value(self.dict, self.json, self.form, self.args)
                 if parameter.name == "no_log":
                     self.no_log = parameter.value
                 else:
@@ -164,11 +166,7 @@ class Route:
             except Exception as ex:
                 # TODO: enhance
                 message = traceback.format_exc()
-                warning = 0
-                if isinstance(ex, AlphaException):
-                    message = ex.description
-                    warning = ex.warning
-                self.set_error(status=message, warning=warning)
+                self.set_error(status=message, ex=ex)
                 self.ex = ex
                 return
 
@@ -267,7 +265,7 @@ class Route:
             else description_from_status(self.returned.status)
         )
 
-    def get_status(self) -> Tuple[str, str]:
+    def get_status(self) -> tuple[str, str]:
         return (self.returned.status, self.returned.status_description)
 
     def timeout(self):
@@ -276,9 +274,13 @@ class Route:
         self.returned.status_code = 524
         self.returned.error = 1
 
-    def access_denied(self):
+    def access_denied(self, description: str | None = None):
         self.returned.status = "unauthorized"
-        self.returned.status_description = description_from_status(self.returned.status)
+        self.returned.status_description = (
+            description_from_status(self.returned.status)
+            if description is None
+            else description
+        )
         self.returned.token_status = "denied"
         self.returned.error = 1
         self.returned.status_code = 401
@@ -298,27 +300,39 @@ class Route:
 
     def set_error(
         self,
-        status: typing.Union[str, AlphaException] = "error",
-        description: str = None,
+        status: str | AlphaException = "error",
+        description: str | None = None,
         warning: int = 0,
+        error_code: int = 520,
+        ex=None,
     ):
         log_fct = self.log.error if warning == 0 else self.log.warning
+
         if isinstance(status, AlphaException):
-            warning = status.warning
-            description = status.description
+            ex = status
+            status = "error"
+        if type(status) == "str" and ("Traceback" in status or "\n" in status):
+            description = status
+            status = "error"
+        if isinstance(ex, AlphaException):
+            warning = ex.warning
+            description = ex.description
             status = (
-                status.name
-                if status.name.lower() != status.description.lower()
-                else "exception"
+                ex.name if ex.name.lower() != ex.description.lower() else "exception"
             )
+            # description += "Exception\n" + "\n".join(ex.traces)
             if not self.no_log:
                 log_fct(f"{status} - {description}", level=2)
         elif not self.no_log:
+            if description is None:
+                description = ""
+            if ex is not None:
+                description += f"\n\n{ex}"
             log_fct(f"{status} - {description}", level=2)
 
         self.mode == "data"
         self.returned.status = str(status)
-        self.returned.status_code = 520 if not warning else 200
+        self.returned.status_code = error_code if not warning else 200
         self.returned.status_description = (
             str(description)
             if description
@@ -329,8 +343,8 @@ class Route:
 
     def set_warning(
         self,
-        status: typing.Union[str, AlphaException] = "warning",
-        description: str = None,
+        status: str | AlphaException = "warning",
+        description: str | None = None,
     ):
         self.set_error(status=status, description=description, warning=1)
 
@@ -365,122 +379,60 @@ class Route:
             and self.parameters.get("per_page", None) is not None
         )
 
-    def get_return(self, forceData=False, return_status=None):
-        default_format = self.api.conf.get("default_format")
-        if default_format is None:
-            default_format = "json"
-        format_ = self.get("format", default=default_format, enable_none=False).lower()
-
-        if self.mode == "html":
-            if "page" in self.data:
-                return render_template(self.data["page"], **self.data["parameters"])
+    def is_pagination(self) -> bool:
+        if self._is_pagination is None:
+            pagination_mode = self.api.db.config.get("pagination_mode", "standard")
+            if pagination_mode == "standard":
+                is_pagination = self.api.db.full_count is not None
             else:
-                return self.data
-        elif self.mode == "print":
-            return self.message
-        elif "get_file" in self.mode:
-            file_path, filename = self.file_to_get
-            if file_path is not None and filename is not None:
-                if not self.no_log:
-                    self.log.info(f"Sending file {filename} from {file_path}")
-                try:
-                    return send_from_directory(
-                        file_path,
-                        filename,
-                        as_attachment="attached" in self.mode,
-                    )
-                except FileNotFoundError:
-                    self.set_error("missing_file")
-            else:
-                self.set_error("missing_file")
-
-        if isinstance(self.data, Exception):
-            if hasattr(self.data, "msg"):
-                self.set_error(self.data.msg, self.data.msg)
-            elif hasattr(self.data, "args"):
-                self.set_error(self.data.args[0], self.data.args[0])
-            self.data = {}
-
-        self.returned.data = {}
+                is_pagination = self.page is not None and (
+                    py_lib.is_list(self.data)
+                    and len(self.data) == 2
+                    and type(self.data[0]) == list
+                    and type(self.data[1]) == int
+                )
+            self._is_pagination = is_pagination
+            return is_pagination
+        return self._is_pagination
 
+    def set_pagination(self):
+        format_ = self.get_format()
         data = {} if self.data is None else self.data
-        page, per_page = self.get("page", 0), self.get("per_page")
+        self.total, self.total_pages, self.page, self.per_page = (
+            None,
+            None,
+            self.get("page", 0),
+            self.get("per_page", None),
+        )
 
-        pagination_mode = self.api.db.config.get("pagination_mode", "raw")
-        if pagination_mode == "integrated":
-            is_pagination = self.api.db.full_count is not None
-            if is_pagination:
-                per_page = len(data)
-                total = (
+        is_pagination = self.is_pagination()
+        pagination_mode = self.api.db.config.get("pagination_mode", "standard")
+        if pagination_mode == "standard":
+            if is_pagination and py_lib.is_iterable(data):
+                self.per_page = len(data) if self.per_page is None else self.per_page
+                self.total = (
                     self.api.db.full_count if self.api.db.full_count is not None else 0
                 )
-                total_pages = (
-                    int(total / per_page)
-                    if per_page is not None and per_page != 0
+                self.total_pages = (
+                    int(self.total / self.per_page)
+                    if self.per_page is not None and self.per_page != 0
                     else 0
                 )
         else:
-            is_pagination = page is not None and (
-                py_lib.is_list(data)
-                and len(data) == 2
-                and type(data[0]) == list
-                and type(data[1]) == int
-            )
-            if per_page is None and is_pagination:
-                per_page = len(data[0])
+            if self.per_page is None and is_pagination:
+                self.per_page = len(data[0])
 
-            if "pagination" in format_ and is_pagination:
-                total, total_pages = (
+            if not "inpagination" in format_ and is_pagination:
+                self.total, self.total_pages = (
                     data[1] if data[1] is not None else 0,
-                    int(data[1] / per_page) if per_page is not None else 0,
+                    int(data[1] / self.per_page) if self.per_page is not None else 0,
                 )
                 data = data[0]
+            self.data = data
 
-        if not check_format(data):
-            tic = time.perf_counter()
-            data = json_lib.jsonify_data(data)
-            toc = time.perf_counter()
-            elapsed_time = toc - tic
-            print(f"Elapsed time: {elapsed_time:0.4f} seconds for jsonify_data")
-
-        if "table" in format_:
-            data = get_columns_values_output(
-                data, self.get("columns"), header="header" in format_
-            )
-
-        self.returned.data = data
-        status_code = self.returned.status_code
-        if type(self.returned.status) == int:
-            status_code = self.returned.status
-
+    def get_headers(self):
+        format_ = self.get_format()
         headers = {}
-
-        tic = time.perf_counter()
-        if "txt" in format_:
-            route_output = str(data)
-            if (
-                route_output is not None
-                and len(route_output) != 0
-                and route_output[0] == '"'
-            ):
-                route_output = route_output[1:-1]
-        elif "raw" in format_:
-            route_output = jsonify(data)
-        elif "xml" in format_:
-            route_output = (
-                self.returned.to_json() if "raw" in format_ else jsonify(data)
-            )
-            route_output = converter_lib.dict_to_xml(
-                route_output, attr_type=not "no_type" in format_
-            )
-        else:
-            # returned = jsonify(self.returned)
-            route_output = self.returned.to_json()
-
-        toc = time.perf_counter()
-        elapsed_time = toc - tic
-        print(f"Elapsed time: {elapsed_time:0.4f} seconds for conversion")
-
         if "txt" in format_:
             headers["Content-Type"] = "text/txt; charset=utf-8"
         elif "xml" in format_:
@@ -502,28 +454,141 @@ class Route:
         # headers["Access-Control-Allow-Headers"] = "*" # ",".join(headers_content.keys())
 
         # returned = Response(returned, status=returned["status_code"])
+        if self.is_pagination():
+            headers["X-pagination"] = json.dumps(
+                {
+                    "total": self.total,
+                    "total_pages": self.total_pages,
+                    "page": self.page,
+                    "previous_page": self.page - 1 if self.page > 0 else 0,
+                    "next_page": (
+                        self.page + 1
+                        if self.page < self.total_pages
+                        else self.total_pages
+                    )
+                    if (self.total_pages is not None and self.page is not None)
+                    else None,
+                    "per_page": self.per_page,
+                }
+            )
+
+        return headers
+
+    def get_format(self) -> str:
+        default_format = self.api.conf.get("default_format", "standard")
+        format_ = self.get("format", default=default_format, enable_none=False).lower()
+        return format_
+
+    def get_return(self, forceData=False, return_status=None):
+        format_ = self.get_format()
+
+        if self.mode == "html":
+            if "page" in self.data:
+                return render_template(self.data["page"], **self.data["parameters"])
+            else:
+                return self.data
+        elif self.mode == "print":
+            return self.message
+        elif "get_file" in self.mode:
+            directory, filename = self.file_to_get
+            if directory is not None and filename is not None:
+                if not self.no_log:
+                    self.log.info(f"Sending file {filename} from {directory}")
+                try:
+                    return send_from_directory(
+                        directory,
+                        filename,
+                        as_attachment="attached" in self.mode,
+                    )
+                except FileNotFoundError:
+                    self.set_error("missing_file")
+            else:
+                self.set_error("missing_file")
+
+        if isinstance(self.data, Exception):
+            if hasattr(self.data, "msg"):
+                self.set_error(self.data.msg, self.data.msg)
+            elif hasattr(self.data, "args"):
+                self.set_error(self.data.args[0], self.data.args[0])
+            self.data = {}
+
+        self.returned.data = {}
+
+        self.set_pagination()
+
+        data = self.data
+        if "check" in format_ and not check_format(data):
+            tic = time.perf_counter()
+            data = json_lib.jsonify_data(data)
+            toc = time.perf_counter()
+            elapsed_time = toc - tic
+            if DEBUG:
+                print(f"Elapsed time: {elapsed_time:0.4f} seconds for jsonify_data")
+
+        if "table" in format_:
+            data = get_columns_values_output(
+                data, self.get("columns"), header="header" in format_
+            )
+
+        self.returned.data = data
+        status_code = self.returned.status_code
+        if type(self.returned.status) == int:
+            status_code = self.returned.status
+
         tic = time.perf_counter()
-        returned = make_response(route_output, status_code)
+
+        returned = None
+        route_output = (
+            data if not "integrated" in format_ else self.returned
+        )  # if "make" in format_ else ujson.dumps(data)
+
+        if "xml" in format_:
+            route_output = json_lib.jsonify_data(route_output)
+            route_output = converter_lib.dict_to_xml(
+                route_output, attr_type=not "no_type" in format_
+            )
+        elif "txt" in format_:
+            if type(route_output) != str:
+                route_output = json.dumps(route_output, cls=AlphaJSONEncoder)
+            route_output = str(route_output)
+            if (
+                route_output is not None
+                and len(route_output) != 0
+                and route_output[0] == '"'
+            ):
+                route_output = route_output[1:-1]
+        elif "jsonify" in format_:
+            returned = jsonify(route_output)
+        else:
+            route_output = json.dumps(route_output, cls=AlphaJSONEncoder)
+
         toc = time.perf_counter()
         elapsed_time = toc - tic
-        print(f"Elapsed time: {elapsed_time:0.4f} seconds for returned")
+        if DEBUG:
+            print(f"Elapsed time: {elapsed_time:0.4f} seconds for conversion")
 
-        if "pagination" in format_ and is_pagination:
-            headers["X-pagination"] = {
-                "total": total,
-                "total_pages": total_pages,
-                "page": page,
-                "previous_page": page - 1 if page > 0 else 0,
-                "next_page": page + 1 if page < total_pages else total_pages,
-                "per_page": per_page,
-            }
+        tic = time.perf_counter()
+
+        if returned is None:
+            if "make" in format_:
+                returned = make_response(route_output, status_code)
+            else:
+                returned = make_response({}, status_code)
+                returned.set_data(route_output)
+
+        toc = time.perf_counter()
+        elapsed_time = toc - tic
+        if DEBUG:
+            print(f"Elapsed time: {elapsed_time:0.4f} seconds for returned")
+
+        headers = self.get_headers()
 
         for key, value in headers.items():
-            if not "\n" in str(value):
-                returned.headers[key] = value
-            else:
-                returned.headers[key] = json.dumps([str(v) for v in value.split("\n")])
-
+            returned.headers[key] = (
+                ujson.dumps([str(v) for v in value.split("\n")])
+                if "\n" in str(value)
+                else value
+            )
         return returned
 
     def log_user(self, user):
