@@ -1,11 +1,21 @@
-import os, configparser, datetime, copy, jwt, logging, re, itertools, sys, importlib, warnings
-import typing
-import traceback
-from typing import Dict, List
+# MODULES
+import os, jwt, itertools, sys, requests, warnings, traceback, time, socket
+from requests.exceptions import HTTPError
+
+# WSGI
+try:
+    from gevent.pywsgi import WSGIServer
+except:
+    pass
+
+# WERKZEUG
+from werkzeug.debug import DebuggedApplication
+
+# FLASK
 from flask import Flask, jsonify, request, Response, make_response
 from flask_mail import Mail
 
-from ...config.main_configuration import CONFIGURATION
+import flask_monitoringdashboard
 
 with warnings.catch_warnings():
     from flask_marshmallow import Marshmallow
@@ -14,120 +24,64 @@ from flask_statistics import Statistics
 from flask_debugtoolbar import DebugToolbarExtension
 from flask_admin import Admin
 
-from gevent.pywsgi import WSGIServer
-
-from werkzeug.debug import DebuggedApplication
-from werkzeug._reloader import reloader_loops, ReloaderLoop, _find_stat_paths
-import flask_monitoringdashboard
+# CONFIGURATIONS
+from ...config.main_configuration import CONFIGURATION
 
+# LIBS
 from ...libs import (
-    mail_lib,
-    flask_lib,
     io_lib,
     os_lib,
     config_lib,
     secure_lib,
 )
 
+# MODELS
 from ...models.logger import AlphaLogger
 from ...models.main import AlphaException
 from ...models.config import AlphaConfig
 from ...models.json import AlphaJSONEncoder
 from ...models.api import AlphaRequest
 
-from ...utils.time import tic, tac
-
-from . import _utils, _colorations
-
+# LOCAL
+from . import _colorations
+from ._reloaders import reloader_loops
 from ._route import Route
 
 
-class AlphaReloaderLoop(ReloaderLoop):
-    name = "AlphaStat"
+def get_fauth_user(url, access_token, verify=False):
+    from ...models.user import AlphaUser
 
-    def __enter__(self) -> ReloaderLoop:
-        self.mtimes: typing.Dict[str, float] = {}
-        self.sizes: typing.Dict[str, float] = {}
-        self.stats: typing.Dict[str, object] = {}
-        return super().__enter__()
+    # Les headers pour inclure le token dans la requête
+    headers = {"Authorization": f"bearer {access_token}"}
 
-    def run_step(self) -> None:
-        for name in itertools.chain(
-            _find_stat_paths(self.extra_files, self.exclude_patterns)
-        ):
-            try:
-                stats = os.stat(name)
-            except OSError:
-                continue
-            try:
-                mtime = stats.st_mtime
-            except OSError:
-                continue
-            try:
-                size = stats.st_size
-            except OSError:
-                continue
+    user = None
+    # Effectuez la requête HTTP POST
+    try:
+        response = requests.post(f"{url}", headers=headers, verify=verify)
 
-            old_time = self.mtimes.get(name)
-            old_size = self.sizes.get(name)
-            old_stats = self.stats.get(name)
+        # Vérifiez si la réponse est réussie
+        response.raise_for_status()
 
-            if old_time is None:
-                self.mtimes[name] = mtime
-                continue
-            if old_size is None:
-                self.sizes[name] = size
-                continue
-            if old_stats is None:
-                self.stats[name] = stats
-                continue
+        # Parsez le JSON de la réponse
+        user_data = response.json()
+        user_data["mail"] = user_data["email"]
 
-            if size != old_size:
-                print(
-                    f"   > Reloading {name}: mtimes {mtime} > {old_time} sizes {size} / {old_size}"
-                )
-                self.trigger_reload(name)
+        infos = ["short_login", "full_name", "location", "country", "region"]
+        user_data["infos"] = {x: user_data[x] for x in infos}
+        user = AlphaUser.map_from_dict(user_data)
 
-            elif mtime > old_time:
-                print(
-                    f"   > File change {name} {mtime} > {old_time}:\n      {old_stats}\n      {stats}"
-                )
+    except HTTPError as http_err:
+        # Vous pouvez ajouter ici une logique supplémentaire pour gérer les erreurs
+        print(f"HTTP error occurred: {http_err}")
+        print(f"Response content: {response.content}")
 
-
-class AlphaMTimeReloaderLoop(ReloaderLoop):
-    name = "AlphaMTimeSimpleStat"
-
-    def __enter__(self) -> ReloaderLoop:
-        self.mtimes: t.Dict[str, float] = {}
-        return super().__enter__()
-
-    def run_step(self) -> None:
-        for name in itertools.chain(
-            _find_stat_paths(self.extra_files, self.exclude_patterns)
-        ):
-            try:
-                mtime = os.stat(name).st_mtime
-            except OSError:
-                continue
-
-            old_time = self.mtimes.get(name)
-
-            if old_time is None:
-                self.mtimes[name] = mtime
-                continue
-
-            if mtime > old_time:
-                print(
-                    f"   REALOGIND {name=} {mtime=} {old_time=} {self.extra_files=} {self.exclude_patterns=}"
-                )
-                self.trigger_reload(name)
-
-
-reloader_loops["alpha"] = AlphaMTimeReloaderLoop
+    except Exception as err:
+        print(f"Other error occurred: {err}")
+    return user
 
 
 class AlphaFlask(Flask):
-    admin_token: str = None
+    admin_token: str | None = None
 
     def __init__(
         self,
@@ -143,8 +97,7 @@ class AlphaFlask(Flask):
         self.__initiated: bool = False
         self.pid = None
         self.conf = None
-        self.config_path = ""
-        self.port = None
+        self._port = None
 
         self.db = None
         self.db_cnx = {}
@@ -160,7 +113,14 @@ class AlphaFlask(Flask):
 
         self.log: AlphaLogger = core.get_logger(CONFIGURATION.API_LOGGER_NAME)
         self.log_requests: AlphaLogger = core.get_logger(CONFIGURATION.HTTP_LOGGER_NAME)
-        self.set_config(config_name, configuration, root)
+
+        core_api_config = core.config.get_config("api", None)
+        if core_api_config is not None:
+            self.conf = core_api_config
+            self.configuration = core.configuration
+        else:
+            print("   ", config_name, configuration, root, core.configuration)
+            self.set_config(config_name, configuration, root)
 
         confs = self.conf.get("config")
         if confs is not None:
@@ -172,28 +132,15 @@ class AlphaFlask(Flask):
             for key, value in confs.items():
                 self.config[key] = value
 
-    def set_error(
-        self,
-        status: typing.Union[str, AlphaException] = "error",
-        description: str = None,
-        warning: int = 0,
-    ):
-        self.get_current_route().set_error(
-            status=status, description=description, warning=warning
-        )
+    @property
+    def port(self):
+        if self._port is None:
+            self._port = self.conf.get("port")
+        return self._port
 
-    def set_status(self, status="success", description=None):
-        self.get_current_route().set_status(status=status, description=description)
-
-    def get_status(self) -> typing.Tuple[str, str]:
-        return self.get_current_route().get_status()
-
-    def set_warning(
-        self,
-        status: typing.Union[str, AlphaException] = "warning",
-        description: str = None,
-    ):
-        self.set_error(status=status, description=description, warning=1)
+    @port.setter
+    def port(self, value):
+        self._port = int(value) if not isinstance(value, int) else value
 
     def set_config(self, name, configuration=None, root=None):
         self.log.info(
@@ -215,16 +162,42 @@ class AlphaFlask(Flask):
                 "routes_no_log"
             )
 
+    def set_error(
+        self,
+        status: str | AlphaException = "error",
+        description: str | None = None,
+        warning: int = 0,
+    ):
+        self.get_current_route().set_error(
+            status=status, description=description, warning=warning
+        )
+
+    def set_status(self, status="success", description=None):
+        self.get_current_route().set_status(status=status, description=description)
+
+    def get_status(self) -> tuple[str, str]:
+        return self.get_current_route().get_status()
+
+    def set_warning(
+        self,
+        status: str | AlphaException = "warning",
+        description: str | None = None,
+    ):
+        self.set_error(status=status, description=description, warning=1)
+
     def get_parameters(
         self,
         not_none: bool = False,
-        without: List[str] = None,
-        added: Dict[str, object] = None,
-    ) -> Dict[str, object]:
+        without: list[str] = None,
+        added: dict[str, object] = None,
+        with_user_id: bool = False,
+        with_user_permissions: bool = False,
+        with_user: bool = False,
+    ) -> dict[str, object]:
         """Get non private route parameters values as a dict.
 
         Returns:
-            Dict[str, object]: [description]
+            dict[str, object]: [description]
         """
         route = self.get_current_route()
         parameters = route.parameters
@@ -240,14 +213,28 @@ class AlphaFlask(Flask):
         if added is not None:
             for key, value in added.items():
                 parameters_values[key] = value
+
+        if any([with_user_id, with_user, with_user_permissions]):
+            user = self.get_logged_user()
+        if with_user_id:
+            parameters_values["user_id"] = user.id if user is not None else None
+        if with_user:
+            parameters_values["user"] = user
+        if with_user_permissions:
+            parameters_values["user_permissions"] = (
+                user.permissions if user is not None else None
+            )
         return parameters_values
 
     def gets(
         self,
         not_none: bool = False,
-        without: List[str] = None,
-        added: Dict[str, object] = None,
-    ) -> Dict[str, object]:
+        without: list[str] = None,
+        added: dict[str, object] = None,
+        with_user_id: bool = False,
+        with_user_permissions: bool = False,
+        with_user: bool = False,
+    ) -> dict[str, object]:
         return self.get_parameters(**{x: y for x, y in locals().items() if x != "self"})
 
     def is_error(self) -> bool:
@@ -276,48 +263,35 @@ class AlphaFlask(Flask):
 
         uri = self.db_cnx[CONFIGURATION.MAIN_DATABASE_NAME]["cnx"]
         if ":///" in uri:
-            io_lib.ensure_file(uri.split(":///")[1])
-        db_type = self.db_cnx[CONFIGURATION.MAIN_DATABASE_NAME]["type"]
+            io_lib.ensure_file(uri.split(":///")[1].split("?")[0])
+        # db_type = self.db_cnx[CONFIGURATION.MAIN_DATABASE_NAME]["type"]
         self.config["SQLALCHEMY_DATABASE_URI"] = uri
 
-        for key, cf_db in self.db_cnx.items():
-            self.config["SQLALCHEMY_BINDS"] = {
-                x: y["cnx"] for x, y in self.db_cnx.items()
-            }
+        self.config["SQLALCHEMY_BINDS"] = {x: y["cnx"] for x, y in self.db_cnx.items()}
 
         # self.api.config["MYSQL_DATABASE_CHARSET"]           = "utf8mb4"
         # self.api.config["QLALCHEMY_TRACK_MODIFICATIONS"]    = True
         # self.api.config["EXPLAIN_TEMPLATE_LOADING"]         = True
         self.config["UPLOAD_FOLDER"] = self.root_path
+        # self.config["SQLALCHEMY_COMMIT_ON_TEARDOWN"] = False
 
     def run(self, *args, **kwargs):
         self.init_run()
         super().run(*args, **kwargs)
 
     def init(self, encode_rules={}):
-        routes = self.conf.get("routes")
+        """routes = self.conf.get("routes")
         if routes is not None:
             for route in routes:
                 try:
                     print(f"Import route {route}")
                     module = importlib.import_module(route)
                 except Exception as ex:
-                    self.log.critical(f"Cannot import routes from {route}", ex=ex)
+                    self.log.critical(f"Cannot import routes from {route}", ex=ex)"""
         self.cache_dir = self.conf.get("directories/cache")
 
-        if routes is not None:
-            self.log.info(f"Init api with routes: {', '.join(routes)}")
-
-        # check request
-        # ! freeze - dont know why
-        """db              = core.get_database(CONFIGURATION.MAIN_DATABASE_NAME)
-        request_model    = database_lib.get_table(CONFIGURATION.MAIN_DATABASE_NAME, "request")
-        try:
-            self.log.debug("Check <request> table")
-            obj = db.exist(request_model)
-        except:
-            self.log.info("Creating <request> table")
-            request_model.__table__.create(db.engine)"""
+        """if routes is not None:
+            self.log.info(f"Init api with routes: {', '.join(routes)}")"""
 
         # Flask configuration
         # todo: check JWT_SECRET_KEY: mandatory
@@ -365,6 +339,8 @@ class AlphaFlask(Flask):
         # Base.prepare(self.db.engine, reflect=True)
 
     def init_admin_view(self):
+        from ...libs import flask_lib  # TODO: change
+
         views = flask_lib.load_views(self.log)
         endpoints = [x.endpoint for x in views]
 
@@ -383,7 +359,6 @@ class AlphaFlask(Flask):
         if self.__initiated:
             return
         host = self.conf.get("host")
-        self.port = self.conf.get("port")
         self.debug = (
             self.conf.get("debug")
             if not "ALPHA_DEBUG" in os.environ
@@ -430,11 +405,26 @@ class AlphaFlask(Flask):
             )
             server.serve_forever()
         else:
-            reloader_types = ["alpha", "auto", "stat", "watchdog"]
-            reloader_type = self.conf.get("reloader_type", default="auto")
-            if reloader_type not in reloader_types:
+            watched_dirs = self.conf.get(
+                "watched_dirs", default=[self.conf.get("project")]
+            )
+            os.environ["ALPHA_WATCHDIRS"] = ";".join(watched_dirs)
+            reloader_type = self.conf.get("reloader_type", default="alpha")
+            exlude_patterns = self.conf.get("exlude_patterns", default=[])
+            exlude_patterns.extend(
+                [
+                    "*.vscode*",
+                    "*site-packages*",
+                    "*Anaconda3*",
+                    "*Miniconda3*",
+                    "*werkzeug*",
+                    "*AppData*",
+                    "*.zip*",
+                ]
+            )
+            if reloader_type not in reloader_loops.keys():
                 self.log.critical(
-                    f"{reloader_type=} does not exist, it must be in {reloader_types}"
+                    f"{reloader_type=} does not exist, it must be in {list(reloader_loops.keys())}"
                 )
                 return
 
@@ -444,22 +434,14 @@ class AlphaFlask(Flask):
                 debug=self.debug,
                 threaded=threaded,
                 ssl_context=ssl_context,
-                exclude_patterns=[
-                    "*.vscode*",
-                    "*site-packages*",
-                    "*Anaconda3*",
-                    "*Miniconda3*",
-                    "*werkzeug*",
-                    "*AppData*",
-                    "*.zip*",
-                ],
+                exclude_patterns=exlude_patterns,
                 reloader_type=reloader_type,
             )
 
         # except SystemExit:
         #    self.info("API stopped")
 
-    def stop(self, config_path=None):
+    def stop(self, config_path=None):  # TODO: change
         if config_path is None:
             config_path = self.config_path
         if self.config_path is None:
@@ -516,87 +498,27 @@ class AlphaFlask(Flask):
         if self.log is not None:
             self.log.warning(message, level=4)
 
-    def send_mail(self, mail_config, parameters_list, db, sender=None):
-        # Configuration
-        main_mail_config = self.get_config(["mails"])
-        config = self.get_config(["mails", "configurations", mail_config])
-        if config is None or type(config) != dict:
-            self.log.error(
-                'Missing "%s" mail configuration in "%s"' % (config, self.config_path)
-            )
-            return False
-
-        # Parameters
-        root_config = copy.copy(main_mail_config["parameters"])
-        for key, parameter in main_mail_config["parameters"].items():
-            root_config[key] = parameter
-
-        # Sender
-        if sender is None:
-            if "sender" in config:
-                sender = config["sender"]
-            elif "sender" in main_mail_config:
-                sender = main_mail_config["sender"]
-            elif "sender" in main_mail_config["parameters"]:
-                sender = main_mail_config["parameters"]["sender"]
-            else:
-                self.set_error("sender_error")
-                return False
-
-        full_parameters_list = []
-        for parameters in parameters_list:
-            # root_configuration          = copy.deepcopy(self.get_config())
-
-            parameters_config = {}
-            if "parameters" in config:
-                parameters_config = copy.deepcopy(config["parameters"])
-
-            full_parameters = {"title": config["title"]}
-
-            _utils.fill_config(parameters_config, source_configuration=parameters)
-            _utils.fill_config(root_config, source_configuration=parameters)
-            _utils.fill_config(root_config, source_configuration=parameters_config)
-            _utils.fill_config(parameters_config, source_configuration=root_config)
-            _utils.fill_config(parameters, source_configuration=parameters_config)
-            _utils.fill_config(parameters, source_configuration=root_config)
-
-            _utils.merge_configuration(
-                full_parameters, source_configuration=root_config, replace=True
-            )
-            _utils.merge_configuration(
-                full_parameters, source_configuration=parameters_config, replace=True
-            )
-            _utils.merge_configuration(
-                full_parameters, source_configuration=parameters, replace=True
-            )
-
-            full_parameters_list.append(full_parameters)
-
-        mail_lib.KEY_SIGNATURE = (self.get_config("mails/key_signature"),)
-        valid = mail_lib.send_mail(
-            mail_path=self.get_config("mails/path"),
-            mail_type=config["mail_type"],
-            parameters_list=full_parameters_list,
-            sender=sender,
-            db=db,
-            log=self.log,
-        )
-        if not valid:
-            self.set_error("mail_error")
-        return valid
-
-    def get_current_route(self) -> Route:
+    def get_current_route(self) -> Route | None:
         """Return the current route.
 
         Returns:
             Route: [description]
         """
         request_uuid = AlphaRequest.get_uuid()
-
+        try:
+            full_path = request.full_path
+        except:
+            return None
         if request_uuid in self.routes_objects:
             return self.routes_objects[request_uuid]
 
-        default_route = Route(request_uuid, request.full_path, [], request)
+        default_route = Route(
+            uuid=request_uuid,
+            route=full_path,
+            parameters=[],
+            request_state=request,
+            api=self,
+        )
         if request_uuid not in self.routes_objects:
             self.log.critical(f"Cannot get route for {request_uuid}")
             return default_route
@@ -626,7 +548,14 @@ class AlphaFlask(Flask):
             if secure_lib.check_cry_operation_code(admin_parameter, admin_password):
                 return True
 
-        ip = request.remote_addr
+        try:
+            ip = request.remote_addr
+        except:
+            ## getting the hostname by socket.gethostname() method
+            hostname = socket.gethostname()
+            ## getting the IP address using socket.gethostbyname() method
+            ip_address = socket.gethostbyname(hostname)
+            ip = "127.0.0.1"
         admins_ips = self.conf.get("admins/ips")
         if admins_ips and (ip in admins_ips or f"::ffff:{ip}" in admins_ips):
             return True
@@ -678,6 +607,8 @@ class AlphaFlask(Flask):
         return user
 
     def get_logged_user(self, algorithms=["HS256"]):
+        from ...apis.users.users import LOGIN_MODE
+
         from ...utils.api import ADMIN_USER_ID_PUBLIC, ADMIN_USER_NAME_PUBLIC
         from ...models.user import AlphaUser
 
@@ -689,10 +620,16 @@ class AlphaFlask(Flask):
             AlphaRequest.get_token() if self.admin_token is None else self.admin_token
         )
 
-        su_user_id = self[ADMIN_USER_ID_PUBLIC.name]
-        su_user_name = self[ADMIN_USER_NAME_PUBLIC.name]
+        su_user_id, su_user_name = (
+            self[ADMIN_USER_ID_PUBLIC.name],
+            self[ADMIN_USER_NAME_PUBLIC.name],
+        )
 
-        if (
+        if LOGIN_MODE.lower() == "fauth":
+            auth_url = self.conf.get("auth/fauth_url")
+            verify = self.conf.get("auth/fauth_verify", False)
+            user = get_fauth_user(auth_url, token, verify)
+        elif (
             self.is_admin(check_logged_user=False)
             and su_user_id is not None
             or su_user_name is not None
@@ -702,11 +639,13 @@ class AlphaFlask(Flask):
             try:
                 user_data_token = jwt.decode(token, secret_key, algorithms=algorithms)
             except:
+                #self.set_error("wrong_token")
                 return None
 
             from ...libs import user_lib  # todo: modify
 
             user = user_lib.get_user_data_by_id(user_data_token["id"], True)
+
             if user is None and self.admin_token == token:
                 return AlphaUser.map_from_token(user_data_token)  # TODO: modify
 
