@@ -1,11 +1,7 @@
-import os, json, inspect, copy, sys, socket, re, platform, getpass
+import os, ujson, copy, sys, socket, re, platform, getpass
 import numpy as np
-from typing import List, Dict
-from collections.abc import Iterable
-from sqlalchemy.orm import scoped_session, sessionmaker
-from sqlalchemy.ext.declarative import declarative_base
 
-from ...config.main_configuration import CONFIGURATION
+from collections.abc import Iterable
 
 from ._utils import *
 
@@ -14,11 +10,41 @@ from ..main._exception import EXCEPTIONS
 from ..logger import AlphaLogger
 from ..main import AlphaException
 
-from ...libs import converter_lib, sql_lib, io_lib
+from ...libs import io_lib
 
 _CONFIGURATIONS = {}
 
 
+def replace_fct_matchs(string: str) -> str:
+    if type(string) != str:
+        return string
+    for fct in ["env"]:
+        matchs = re.findall(f"(\${fct}\(([^\$]+)\))", string)
+        if len(matchs) == 1 and len(matchs[0]) == 2:
+            match fct:
+                case "env":
+                    value = os.environ.get(matchs[0][1], None)
+                    if string == matchs[0][0]:
+                        return value
+                    else:
+                        string = string.replace(matchs[0][0], str(value))
+    return string
+
+
+def replace_fcts(d):
+    for key, value in d.items():
+        if isinstance(value, dict):
+            replace_fcts(value)
+        elif isinstance(value, list):
+            for i, item in enumerate(value):
+                if isinstance(item, dict):
+                    replace_fcts(item)
+                elif isinstance(item, str):
+                    value[i] = replace_fct_matchs(item)
+        else:
+            d[key] = replace_fct_matchs(value)
+
+
 def exec_parameters(d):
     if isinstance(d, dict):
         for k, v in d.items():
@@ -39,25 +65,28 @@ class AlphaConfig(AlphaClass):
     def __init__(
         self,
         name: str = "config",
-        filepath: str = None,
-        root: str = None,
-        filename: str = None,
-        log: AlphaLogger = None,
-        configuration: str = None,
+        filepath: str | None = None,
+        root: str | None = None,
+        filename: str | None = None,
+        log: AlphaLogger | None = None,
+        configuration: str | None = None,
         data: dict = None,
         origin=None,
         core=None,
         core_configuration=None,
-        reserved: List[str] = [],
-        required: List[str] = [],
-        user: str = None,
-        default: str = None,
+        reserved: list[str] = [],
+        required: list[str] = [],
+        user: str | None = None,
+        default: str | None = None,
     ):
         if hasattr(self, "tmp"):
             return
 
         if default is not None and os.path.isfile(default):
-            default_config = AlphaConfig(f"default_{name}", filepath=default,)
+            default_config = AlphaConfig(
+                f"default_{name}",
+                filepath=default,
+            )
             data = data if data is not None else {}
             merge_configuration(
                 data, source_configuration=default_config.data, replace=False
@@ -114,7 +143,7 @@ class AlphaConfig(AlphaClass):
         if not self.loaded:
             with open(self.filepath, encoding="utf-8") as json_data_file:
                 try:
-                    self.data_origin = json.load(json_data_file)
+                    self.data_origin = ujson.load(json_data_file)
                 except Exception as ex:
                     print(f"Configuration file {self.filepath} is invalid: {ex}")
                     exit()
@@ -173,9 +202,6 @@ class AlphaConfig(AlphaClass):
             self.__check_required()
             self.__configure()
 
-    """def set_data(self,value,paths=[]):
-        ensure_path(self.data_origin,paths,value=value)"""
-
     def _set_configuration(self):
         if "configurations" in self.data_origin:
             configurations = {
@@ -232,6 +258,7 @@ class AlphaConfig(AlphaClass):
             merge_configuration(config, values, replace=True)
             del config[key]
 
+        replace_fcts(config)
         self.data = self._replace_parameters(config)
 
         self.data = exec_parameters(self.data)
@@ -421,10 +448,10 @@ class AlphaConfig(AlphaClass):
     def get(
         self,
         path=[],
-        root: bool = True,
         default=None,
+        root: bool = True,
         force_exit: bool = False,
-        configuration: str = None,
+        configuration: str | None = None,
         type_=None,
         required: bool = False,
     ):
@@ -538,7 +565,7 @@ class AlphaConfig(AlphaClass):
         self.__clean()
 
         with open(self.filepath, "w", encoding="utf-8") as json_data_file:
-            json.dump(
+            ujson.dump(
                 self.data_origin,
                 json_data_file,
                 sort_keys=True,
@@ -680,7 +707,7 @@ class AlphaConfigurations(object):
         return cls._instance
 
     def __init__(self):
-        self._configurations: Dict[str, AlphaConfig] = {}
+        self._configurations: dict[str, AlphaConfig] = {}
         if os.path.exists(self._name):
             loaded_configurations = io_lib.unarchive_object(self._name)
             if type(loaded_configurations) == dict:
@@ -746,7 +773,7 @@ class AlphaConfigurations(object):
         try:
             self._configurations[key] = dataset
         except:
-            self._configurations: Dict[str, object] = {key: dataset}
+            self._configurations: dict[str, object] = {key: dataset}
         io_lib.archive_object(self._configurations, self._name)
 
     def is_configured(self, config) -> bool:
