@@ -1,4 +1,4 @@
-import datetime, typing
+import datetime
 
 from sqlalchemy import (
     Table,
@@ -17,7 +17,7 @@ from sqlalchemy.sql.elements import BinaryExpression
 
 from .utils import get_schema
 from ..main import AlphaClass
-from ...libs import flask_lib, date_lib
+from ...libs import date_lib
 from ..main import AlphaException
 
 
@@ -85,7 +85,7 @@ class AlphaTable(AlphaClass):
         return repr(self)
 
     def to_json(
-        self, relationship: bool = True, disabled_relationships: typing.List[str] = None
+        self, relationship: bool = True, disabled_relationships: list[str] = None
     ):
         class_obj = self.__class__
         results_json = {}
@@ -114,6 +114,12 @@ class AlphaTable(AlphaClass):
         }
         return self.__init__(**kwargs)
 
+    @classmethod
+    def init_from_model(class_obj, model):
+        columns = class_obj.get_columns()
+        kwargs = {x: y for x, y in model.__dict__.items() if x in columns}
+        return class_obj(**kwargs)
+
     @classmethod
     def instantiate(class_obj, *args, **kwargs):
         """if type(values) == dict:
@@ -135,7 +141,7 @@ class AlphaTable(AlphaClass):
     def get_schema(
         class_obj,
         relationship: bool = True,
-        disabled_relationships: typing.List[str] = None,
+        disabled_relationships: list[str] = None,
     ):
         disabled_relationships = disabled_relationships or []
         """if (
@@ -186,7 +192,7 @@ class AlphaTable(AlphaClass):
                 continue  # TODO: modify
 
     @classmethod
-    def get_columns(class_obj) -> typing.List[str]:
+    def get_columns(class_obj) -> list[str]:
         columns_names = []
 
         for col in class_obj.__table__.columns:
@@ -198,7 +204,7 @@ class AlphaTable(AlphaClass):
         return columns_names
 
     @classmethod
-    def get_model(class_obj) -> typing.Dict[str, str]:
+    def get_table_model_structure(class_obj) -> dict[str, str]:
         columns_names = []
         columns = []
 
@@ -209,14 +215,18 @@ class AlphaTable(AlphaClass):
             if hasattr(col, "name"):
                 columns_names.append(col.name)
 
-        for key, value in class_obj.__dict__.items():
+        class_dict = {x: y for x, y in class_obj.__dict__.items()}
+        for key, value in class_dict.items():
             if hasattr(value, "expression"):
                 if hasattr(value.expression, "name"):
                     if value.expression.name in columns_names:
-                        column_model = value.to_json()
-                        if not hasattr(value, "_proxy_key"):
-                            column_model["key"] = key
-                        columns.append(column_model)
+                        if hasattr(value, "to_json"):
+                            column_model = value.to_json()
+                            if not hasattr(value, "_proxy_key"):
+                                column_model["key"] = key
+                            columns.append(column_model)
+                        else:
+                            print(f"Cannot convert {value}")
         return columns
 
 
@@ -284,7 +294,7 @@ class AlphaInteger(TypeDecorator):
 """
     
 # creating class dynamically 
-def get_model(db):
+def get_table_model(db):
     models = {}
     
     name = "logs"
