@@ -1,9 +1,9 @@
-
 from collections.abc import MutableMapping
 
+
 class Row(MutableMapping):
     """A dictionary that applies an arbitrary key-altering
-       function before accessing the keys"""
+    function before accessing the keys"""
 
     def __init__(self, *args, **kwargs):
         self.store = dict()
@@ -34,7 +34,10 @@ class Row(MutableMapping):
 
     def show(self):
         for k, v in self.store.items():
-            print('   {:20} {}'.format(k,v))
+            print("   {:20} {}".format(k, v))
 
     def keys(self):
         return list(super().keys())
+
+    def to_json(self):
+        return self.store
