@@ -1,3 +1,7 @@
+# MODULES
+import datetime
+
+# SQLALCHEMY
 from sqlalchemy import (
     Table,
     Column,
@@ -17,6 +21,7 @@ from sqlalchemy.orm import relationship, backref
 from sqlalchemy.sql.sqltypes import Boolean
 from marshmallow import Schema, fields
 
+# MODELS
 from .models import (
     AlphaTable,
     AlphaTableId,
@@ -25,14 +30,14 @@ from .models import (
     AlphaTableIdPrimary,
     AlphaTableUpdateDate,
 )
-import datetime
 
+# CORE
 from core import core
 
 db = core.db
 ma = core.ma
 
-BIND = "USERS"
+BIND = core.config.get("user_mgt/bind", "USERS")
 
 
 class UserSession(db.Model, AlphaTableUpdateDate):
@@ -44,18 +49,26 @@ class UserSession(db.Model, AlphaTableUpdateDate):
     ip = AlphaColumn(String(50))
     expire = AlphaColumn(DateTime)
 
+
 class UserRole(db.Model, AlphaTableUpdateDate):
     __bind_key__ = BIND
     __tablename__ = "user_role"
 
     user_id = AlphaColumn(
-        Integer, ForeignKey("user.id"), nullable=False, primary_key=True
+        Integer,
+        ForeignKey("user.id", ondelete="CASCADE"),
+        nullable=False,
+        primary_key=True,
     )
     role_name = AlphaColumn(
-        String(200), ForeignKey("role.name"), nullable=False, primary_key=True
+        String(200),
+        ForeignKey("role.name", ondelete="CASCADE"),
+        nullable=False,
+        primary_key=True,
     )
     activated = AlphaColumn(Boolean, default=True)
-    
+
+
 class User(db.Model, AlphaTable):
     __bind_key__ = BIND
     __tablename__ = "user"
@@ -67,6 +80,7 @@ class User(db.Model, AlphaTable):
     pass_reset_token = AlphaColumn(String(100))
     pass_reset_token_exp = AlphaColumn(DateTime)
     telegram_id = AlphaColumn(String(100))
+    # icon = AlphaColumn(String(100))
     role = AlphaColumn(Integer)
 
     expire = AlphaColumn(DateTime)
@@ -78,10 +92,30 @@ class User(db.Model, AlphaTable):
         DateTime, default=datetime.datetime.now, onupdate=datetime.datetime.now
     )
 
-    roles = relationship("Role", secondary="user_role", cascade="all,delete",)
+    roles = relationship(
+        "Role",
+        secondary=UserRole.__table__,
+        cascade="all, delete",
+    )
 
 
+class RolePermission(db.Model, AlphaTableUpdateDate):
+    __bind_key__ = BIND
+    __tablename__ = "role_permission"
 
+    role_name = AlphaColumn(
+        String(200),
+        ForeignKey("role.name", ondelete="CASCADE"),
+        nullable=False,
+        primary_key=True,
+    )
+    permission_key = AlphaColumn(
+        String(200),
+        ForeignKey("permission.key", ondelete="CASCADE"),
+        nullable=False,
+        primary_key=True,
+    )
+    activated = AlphaColumn(Boolean, default=True)
 
 
 class Role(db.Model, AlphaTableUpdateDate):
@@ -92,7 +126,10 @@ class Role(db.Model, AlphaTableUpdateDate):
     description = AlphaColumn(String(1000))
     activated = AlphaColumn(Boolean, default=True)
 
-    id_app = AlphaColumn(Integer, ForeignKey("application.id"),)
+    id_app = AlphaColumn(
+        Integer,
+        ForeignKey("application.id", ondelete="CASCADE"),
+    )
 
     application = relationship(
         "Application",
@@ -100,23 +137,12 @@ class Role(db.Model, AlphaTableUpdateDate):
     )
 
     permissions = relationship(
-        "Permission", secondary="role_permission", cascade="all,delete",
+        "Permission",
+        secondary=RolePermission.__table__,
+        cascade="all,delete",
     )
 
 
-class RolePermission(db.Model, AlphaTableUpdateDate):
-    __bind_key__ = BIND
-    __tablename__ = "role_permission"
-
-    role_name = AlphaColumn(
-        String(200), ForeignKey("role.name"), nullable=False, primary_key=True
-    )
-    permission_key = AlphaColumn(
-        String(200), ForeignKey("permission.key"), nullable=False, primary_key=True
-    )
-    activated = AlphaColumn(Boolean, default=True)
-
-
 class Permission(db.Model, AlphaTableUpdateDate):
     __bind_key__ = BIND
     __tablename__ = "permission"
@@ -125,7 +151,9 @@ class Permission(db.Model, AlphaTableUpdateDate):
     description = AlphaColumn(String(1000))
     activated = AlphaColumn(Boolean, default=True)
 
-    roles = relationship("Role", secondary="role_permission")
+    roles = relationship(
+        "Role", secondary=RolePermission.__table__, cascade="all,delete"
+    )  # TODO: remove overlaps="permissions" and fix the warning
 
 
 class Notification(db.Model, AlphaTableUpdateDate):
@@ -133,7 +161,10 @@ class Notification(db.Model, AlphaTableUpdateDate):
     __tablename__ = "notification"
 
     user_id = AlphaColumn(
-        Integer, ForeignKey("user.id"), nullable=False, primary_key=True
+        Integer,
+        ForeignKey("user.id", ondelete="CASCADE"),
+        nullable=False,
+        primary_key=True,
     )
     user = relationship(
         "User",
@@ -151,7 +182,7 @@ class Application(db.Model, AlphaTableUpdateDate):
     __bind_key__ = BIND
     __tablename__ = "application"
 
-    id = Column(Integer, primary_key=True)
-    name = Column(String(20), nullable=False)
+    id = AlphaColumn(Integer, primary_key=True)
+    name = AlphaColumn(String(20), nullable=False)
 
-    description = Column(String(255))
+    description = AlphaColumn(String(255))
