@@ -1,12 +1,18 @@
-import importlib, json, typing
+import importlib, datetime
 
 from marshmallow import Schema
 from marshmallow import fields as cfields
 from marshmallow_sqlalchemy import ModelConverter, fields
 
-from ...config.main_configuration import CONFIGURATION
+from sqlalchemy.orm import (
+    RelationshipProperty,
+    ColumnProperty,
+    noload,
+    with_loader_criteria,
+    class_mapper,
+)
 
-from ..logger import AlphaLogger
+from ...config.main_configuration import CONFIGURATION
 
 SCHEMAS = {}
 MODULES = {}
@@ -33,7 +39,7 @@ def __get_nested_schema(mapper, parent=None):
 
 
 def get_auto_schema(
-    model, relationship: bool = True, disabled_relationships: typing.List[str] = []
+    model, relationship: bool = True, disabled_relationships: list[str] = []
 ):
     from core import core
 
@@ -122,11 +128,11 @@ def __pdr_flat(pdr, content="", level=1):
 def generate_schema(
     class_obj,
     relationship: bool = True,
-    disabled_relationships: typing.List[str] = None,
-    parents: typing.List[str] = None,
+    disabled_relationships: list[str] | None = None,
+    parents: list[str] | None = None,
 ):
     parents = parents or []
-    disabled_relationships = disabled_relationships or []
+    disabled_relationships = disabled_relationships or {}
     disabled_relationships = __pdr_list_to_dict(disabled_relationships)
     disabled_relationships_keys = __pdr_flat(disabled_relationships)
 
@@ -174,7 +180,10 @@ def generate_schema(
         elif hasattr(value, "entity") and relationship:
             skip, disabled_relationships_child = False, []
 
-            for (disabled_relationship, childs,) in disabled_relationships.items():
+            for (
+                disabled_relationship,
+                childs,
+            ) in disabled_relationships.items():
                 if skip:
                     continue
 
@@ -243,9 +252,9 @@ def get_schema(
     class_obj,
     default: bool = False,
     relationship: bool = True,
-    disabled_relationships: typing.List[str] = None,
+    disabled_relationships: list[str] | None = None,
 ):
-    """ Get Schema for a model
+    """Get Schema for a model
 
     Args:
         class_obj ([type]): [description]
@@ -254,7 +263,7 @@ def get_schema(
     Returns:
         [type]: [description]
     """
-    disabled_relationships = disabled_relationships or []
+    disabled_relationships = disabled_relationships or {}
 
     schema_name = f"{class_obj.__name__}Schema"
 
@@ -277,3 +286,48 @@ def get_schema(
         disabled_relationships=disabled_relationships,
     )
     return generated_schema
+
+
+def instance_to_dict(instance, mapped: list | None = None):
+    if mapped is None:
+        mapped = []
+
+    def process_value(value):
+        if isinstance(value, datetime.datetime):
+            value = value.isoformat()
+        return value
+
+    result = {
+        key: process_value(value)
+        for key, value in instance.__dict__.items()
+        if key != "_sa_instance_state"
+    }
+
+    mapped.append(instance.__class__)
+    relationships = class_mapper(instance.__class__).relationships
+
+    for prop in relationships:
+        related_instances = getattr(instance, prop.key)
+        if related_instances is None:
+            continue
+
+        if related_instances.__class__ in mapped:
+            continue
+
+        if prop.uselist:
+            result[prop.key] = [
+                instance_to_dict(related_instance, mapped)
+                for related_instance in related_instances
+            ]
+        else:
+            result[prop.key] = instance_to_dict(related_instances, mapped)
+
+    return result
+
+
+def nested_noload(query, disabled_relationships):
+    options = [
+        with_loader_criteria(relationship, lambda _: False)
+        for relationship in disabled_relationships
+    ]
+    return query.options(*options)
