@@ -9,9 +9,9 @@ import numpy as np
 
 from _collections_abc import dict_keys
 
-from flask.json import JSONEncoder
+from json import JSONEncoder
 
-from sqlalchemy.exc import OperationalError
+import sqlalchemy
 
 
 class AlphaJSONEncoder(JSONEncoder):
@@ -34,6 +34,7 @@ class AlphaJSONEncoder(JSONEncoder):
         self.rules[decimal.Decimal] = lambda o: str(o)
         self.rules[enum.Enum] = lambda o: str(o)
         self.rules[str] = lambda o: str(o)
+        self.rules[type(sqlalchemy.null())] = lambda o: "null()"
         # self.rules[NameError] = lambda o:str(o)
         # self.rules[ValueError] = lambda o:str(o)
         # self.rules[TypeError] = lambda o:str(o)
@@ -42,26 +43,24 @@ class AlphaJSONEncoder(JSONEncoder):
         # self.rules[str] = lambda o:str(o)
 
     def default(self, o):  # pylint: disable=E0202
-        if "<class " in str(o):
-            return str(o)
-
+        if hasattr(o, "to_json"):
+            return o.to_json()
+        for key_type, fct in self.rules.items():
+            if isinstance(o, key_type):
+                returned_value = fct(o)
+                return returned_value
         try:
-            if hasattr(o, "to_json"):
-                return o.to_json()
-            for key_type, fct in self.rules.items():
-                if isinstance(o, key_type):
-                    returned_value = fct(o)
-                    return returned_value
+            iterable = iter(o)
+        except:
+            iterable = None
+        if iterable is not None:
             try:
-                iterable = iter(o)
-            except:
-                iterable = None
-        except TypeError as ex:
-            print(f"Cannot convert {o} to json: {ex}")
-        else:
-            if iterable is not None:
                 return list(iterable)
+            except:
+                return o
 
+        """if "<class " in str(o):
+            return str(o)"""
         try:
             return JSONEncoder.default(self, o=o)
         except:
@@ -71,6 +70,10 @@ class AlphaJSONEncoder(JSONEncoder):
             except:
                 return o
 
+        """try:
+        except TypeError as ex:
+            print(f"Cannot convert {o} to json: {ex}")"""
+
         """results_json = {}
         if hasattr(model,"schema"):
             schema          = model.get_schema()
