@@ -1,32 +1,29 @@
+# MODULS
 import dataclasses, traceback
 import enum, inspect, operator, re, copy
-from dataclasses import dataclass, field, is_dataclass, _MISSING_TYPE
+
+from dataclasses import dataclass, field, _MISSING_TYPE, asdict
 from collections import OrderedDict
 
-from typing import Dict, Iterable, List, Optional
-from typing_extensions import Annotated
-
+# LOGGER
 from ..logger import AlphaLogger
+
+# LIBS
 from ...libs import py_lib, json_lib, string_lib
 
+# LOCALS
+from ._enum import MappingMode
 from ._exception import AlphaException
 
+# DEBUG MODE ONLY: be careful it could drastically slow down the system
 FORCE_AUTO_MAP = False
 
 
-class AlphaEnum(enum.Enum):
-    def __str__(self):
-        return str(self.value)
-
-    def to_json(self):
-        return str(self.value)
-
-
 class AlphaMappingAttribute:
     def __init__(
         self,
         name: str,
-        key: str = None,
+        key: str | None = None,
         fct: object = None,
         required: bool = False,
         flat: bool = False,
@@ -38,7 +35,7 @@ class AlphaMappingAttribute:
         self.flat = flat
 
 
-def get_score_dict_best_match(score_dict: Dict[str, int]) -> str:
+def get_score_dict_best_match(score_dict: dict[str, int]) -> str:
     max_keys = [
         x
         for x, y in score_dict.items()
@@ -51,36 +48,77 @@ def get_score_dict_best_match(score_dict: Dict[str, int]) -> str:
     return best_matchs[0]
 
 
+def is_auto_map(dataclass_type, mapping_mode: MappingMode | str | None = None) -> bool:
+    if not hasattr(dataclass_type, "auto_map_from_dict"):
+        return False
+    if dataclass_type.get_mapping_mode() is not None and mapping_mode is None:
+        mapping_mode = dataclass_type.get_mapping_mode()
+    elif mapping_mode is None:
+        mapping_mode = MappingMode.STRICT
+
+    mapping_mode_str = str(mapping_mode)
+    if type(mapping_mode) != str:
+        mapping_mode_str: str = str(mapping_mode.value)
+    mapping_mode_str = mapping_mode_str.lower()
+
+    match mapping_mode_str:
+        case MappingMode.AUTO.value:
+            return True
+    return False
+
+
 class AlphaClass:
-    def __init__(self, *args, log: AlphaLogger = None, **kwargs):
-        self.init_args: dict = {"args": args, "kwargs": kwargs}
-        self.children: list = []
+    _map: MappingMode | None = None
+    _map_keys: dict[str, str] | None = None
+    _no_map: list[str] | None = None
 
-        self.log: AlphaLogger = log
-
-        self.__piles: Dict[str, List[str]] = {}
+    def __init__(self, *args, log: AlphaLogger | None = None, **kwargs):
+        self._init_args: dict = {"args": args, "kwargs": kwargs}
+        self._children: list = []
+        self._logger = log
+        self._piles: dict[str, list[str]] = {}
 
     def make_child(self, child_cls, *args, **kwargs):
         if args is None:
-            args = self.init_args["args"]
+            args = self._init_args["args"]
         if kwargs is None:
-            kwargs = self.init_args["kwargs"]
+            kwargs = self._init_args["kwargs"]
         child = child_cls(self, *args, **kwargs)
-        self.children.append(child)
+        self._children.append(child)
         return child
 
-    def get_attributes(self):
-        return py_lib.get_attributes(self)
+    def get_children(self):
+        return self._children
+
+    @classmethod
+    def get_mapping_mode(class_):
+        return class_._map
 
     def to_json(
-        self, rejected: list = None, not_none: bool = False, not_empty: bool = False
+        self,
+        columns: list | None = None,
+        rejected: list | None = None,
+        not_none: bool = False,
+        not_empty: bool = False,
     ):
-        d_rejected = ["_AlphaClass__piles", "init_args", "children", "log"]
+        """d_rejected = [
+            "_map",
+            "_map_keys",
+            "_no_map",
+            "_piles",
+            "_init_args",
+            "_children",
+            "_log",
+        ]
         if rejected is not None:
-            d_rejected.extend(rejected)
+            d_rejected.extend(rejected)"""
         dict_output = {}
-        for x, y in self.get_attributes().items():
-            if x in d_rejected:
+        for x, y in self.__dict__.items():
+            if rejected and x in rejected:
+                continue
+            if columns is not None and x not in columns:
+                continue
+            if x.startswith("_"):
                 continue
             if not_none and y is None:
                 continue
@@ -93,12 +131,12 @@ class AlphaClass:
         return dict_output
 
     def __log(self, stack, message, ex=None, out: bool = False):
-        if not stack in self.__piles:
-            self.__piles[stack] = []
-        pile = self.__piles[stack]
+        if not stack in self._piles:
+            self._piles[stack] = []
+        pile = self._piles[stack]
 
-        if self.log is not None:
-            method = getattr(self.log, stack)
+        if self._logger is not None:
+            method = getattr(self._logger, stack)
             if len(pile) != 0:
                 for deb in pile:
                     method(deb, ex=ex)
@@ -107,7 +145,7 @@ class AlphaClass:
         else:
             pile.append(message)
             if out:
-                for stack, msg in self.__piles.items():
+                for stack, msg in self._piles.items():
                     print(f"{stack}: {msg}")
                 if ex:
                     print(ex)
@@ -132,7 +170,7 @@ class AlphaClass:
             exit()
 
     def map_from_inputs(
-        self, map_entries: List[AlphaMappingAttribute], inputs: Dict[str, object]
+        self, map_entries: list[AlphaMappingAttribute], inputs: dict[str, object]
     ):
         mapped = []
         for key in inputs:
@@ -175,7 +213,7 @@ def convert_value(value: object, type_):
         )
 
 
-def identify(search_key: str, dict_keys: List[str], dataclass_type):
+def identify(search_key: str, dict_keys: list[str], dataclass_type):
     identified_k = None
     for dict_key in dict_keys:
         if search_key in dict_key or dict_key in search_key:
@@ -194,7 +232,7 @@ def identify(search_key: str, dict_keys: List[str], dataclass_type):
     return identified_k, score
 
 
-def get_association(key: str, associations: Dict[str, str] = {}):
+def get_association(key: str, associations: dict[str, str] = {}):
     associations = {
         x: y if type(associations[x]) != AlphaMappingAttribute else y.key
         for x, y in associations.items()
@@ -209,9 +247,9 @@ def get_association(key: str, associations: Dict[str, str] = {}):
 def full_identify(
     dataclass_type,
     dict_key: str,
-    fields_keys: List[str],
-    associations: Dict[str, str] = {},
-    no_match: List[str] = [],
+    fields_keys: list[str],
+    associations: dict[str, str] = {},
+    no_match: list[str] = [],
 ):
     patterns = {}
     required = False
@@ -271,23 +309,23 @@ def full_identify(
 @dataclass
 class AutoMapping(AlphaClass):
     dataclass_type: object
-    associations: Dict[str, str] = field(default_factory=lambda: {})
-    no_match: List[str] = field(default_factory=lambda: [])
+    associations: dict[str, str] = field(default_factory=lambda: {})
+    no_match: list[str] = field(default_factory=lambda: [])
 
-    object_fields: Dict[str, object] = field(default_factory=lambda: {})
-    dict_fields: Dict[str, object] = field(default_factory=lambda: {})
-    list_fields: Dict[str, object] = field(default_factory=lambda: {})
-    flat_fields: Dict[str, object] = field(default_factory=lambda: {})
+    object_fields: dict[str, object] = field(default_factory=lambda: {})
+    dict_fields: dict[str, object] = field(default_factory=lambda: {})
+    list_fields: dict[str, object] = field(default_factory=lambda: {})
+    flat_fields: dict[str, object] = field(default_factory=lambda: {})
 
-    identified_object_fields: Dict[str, object] = field(default_factory=lambda: {})
-    identified_dict_fields: Dict[str, object] = field(default_factory=lambda: {})
-    identified_list_fields: Dict[str, object] = field(default_factory=lambda: {})
+    identified_object_fields: dict[str, object] = field(default_factory=lambda: {})
+    identified_dict_fields: dict[str, object] = field(default_factory=lambda: {})
+    identified_list_fields: dict[str, object] = field(default_factory=lambda: {})
 
-    matchs_object_fields: Dict[str, object] = field(default_factory=lambda: {})
-    matchs_dict_fields: Dict[str, object] = field(default_factory=lambda: {})
-    matchs_list_fields: Dict[str, object] = field(default_factory=lambda: {})
+    matchs_object_fields: dict[str, object] = field(default_factory=lambda: {})
+    matchs_dict_fields: dict[str, object] = field(default_factory=lambda: {})
+    matchs_list_fields: dict[str, object] = field(default_factory=lambda: {})
 
-    already_assigned: List[str] = field(default_factory=lambda: [])
+    already_assigned: list[str] = field(default_factory=lambda: [])
 
     is_identified: bool = False
     flat: bool = False
@@ -297,14 +335,15 @@ class AutoMapping(AlphaClass):
 
     def __set_fields_types(self):
         for x, y in self.dataclass_type.__dataclass_fields__.items():
-            if py_lib.is_subtype(y.type, List) or py_lib.is_subtype(y.type, list):
+            if py_lib.is_subtype(y.type, list):
                 self.list_fields[x] = y
             elif is_dataclass(y.type):
                 self.dict_fields[x] = y
             else:
                 self.object_fields[x] = y
+        pass
 
-    def identify(self, dict_values: Dict[str, object]):
+    def identify(self, dict_values: dict[str, object]):
         dict_elements = {
             x: y for x, y in dict_values.items() if type(y) == dict or y is None
         }
@@ -367,8 +406,9 @@ class AutoMapping(AlphaClass):
                 if field_key in fields_dict:
                     identified_dict[best_match] = fields_dict[field_key]
                     identified.append(best_match)
-
-    def __identify_flat(self, dict_values: Dict[str, object]):
+        self.is_identified = True 
+        
+    def __identify_flat(self, dict_values: dict[str, object]):
         identified = []
         for dict_key in dict_values.keys():
             identified_k = get_association(dict_key, self.associations)
@@ -401,25 +441,40 @@ class AutoMapping(AlphaClass):
                 self.identified_object_fields[best_match] = self.object_fields[
                     field_key
                 ]
+        self.is_identified = True
 
-    def __map_flat(self, dict_values: Dict[str, object]):
+    def __map_flat(
+        self,
+        dict_values: dict[str, object],
+        mapping_mode: MappingMode | str | None = None,
+    ):
         if not self.is_identified:
             self.__identify_flat(dict_values)
 
         field_values = {}
-        for (key, fd,) in self.identified_object_fields.items():
-            field_values[fd.name] = convert_value(dict_values[key], fd.type)
+        for (
+            key,
+            fd,
+        ) in self.identified_object_fields.items():
+            field_type = py_lib.get_first_non_none_type(fd.type)
+            field_values[fd.name] = convert_value(dict_values[key], field_type)
 
         for fd in self.dict_fields.values():
-            field_values[fd.name] = fd.type.auto_map_from_dict(
+            field_type = py_lib.get_first_non_none_type(fd.type)
+            field_values[fd.name] = field_type.map_from_dict(
                 dict_values={
                     x: y for x, y in dict_values.items() if not x in field_values
-                }
+                },
+                mapping_mode=mapping_mode,
             )
 
         return field_values
 
-    def map(self, dict_values: Dict[str, object]):
+    def map(
+        self,
+        dict_values: dict[str, object],
+        mapping_mode: MappingMode | str | None = None,
+    ):
         if self.flat:
             return self.__map_flat(dict_values)
 
@@ -427,23 +482,38 @@ class AutoMapping(AlphaClass):
             self.identify(dict_values)
 
         field_values = {}
-        for (key, fd,) in self.identified_object_fields.items():
-            field_values[fd.name] = convert_value(dict_values[key], fd.type)
+        for (
+            key,
+            fd,
+        ) in self.identified_object_fields.items():
+            field_type = py_lib.get_first_non_none_type(fd.type)
+            field_values[fd.name] = convert_value(dict_values[key], field_type)
 
-        for (key, fd,) in self.identified_dict_fields.items():
+        for (
+            key,
+            fd,
+        ) in self.identified_dict_fields.items():
+            field_type = py_lib.get_first_non_none_type(fd.type)
             field_values[fd.name] = (
-                fd.type.auto_map_from_dict(dict_values=dict_values[key])
+                field_type.map_from_dict(
+                    dict_values=dict_values[key], mapping_mode=mapping_mode
+                )
                 if dict_values[key] is not None
                 else None
             )
 
-        for (key, fd,) in self.identified_list_fields.items():
+        for (
+            key,
+            fd,
+        ) in self.identified_list_fields.items():
             sub_type = fd.type.__args__[0]
             if hasattr(sub_type, "auto_map_from_dict"):
                 field_values[fd.name] = (
                     [
                         (
-                            sub_type.auto_map_from_dict(dict_values=f)
+                            sub_type.auto_map_from_dict(
+                                dict_values=f, mapping_mode=mapping_mode
+                            )
                             if f is not None
                             else None
                         )
@@ -467,44 +537,49 @@ class AutoMapping(AlphaClass):
         return field_values
 
 
-def convert_value_from_field(field_type, value, automap: bool = False):
+def convert_value_from_field(
+    field_type, value, mapping_mode: MappingMode | str | None = None
+):
+    field_type = py_lib.get_first_non_none_type(field_type)
     if value is None:
         return None
     if is_dataclass(field_type):
-        return field_type.map_from_dict(value, automap=automap)
+        return field_type.map(
+            asdict(value) if is_dataclass(value) else value, mapping_mode=mapping_mode
+        )
+    elif isinstance(value, list) and all(isinstance(item, dict) for item in value):
+        return [convert_value_from_field(field_type, item) for item in value]
     else:
         try:
-            return field_type(value)
+            return convert_value(value, field_type)
         except:
             return value
 
 
 class AlphaDataclass(AlphaClass):
-    def get_fields_dict(self, keys: List[str] = None) -> dict:
+    def get_fields_dict(self, keys: list[str] | None = None) -> dict:
         dct = {}
         # dct['_tname'] = self.__class__.__name__
 
-        for f in dataclasses.fields(self):
-            f_type = f.type
-            value = getattr(self, f.name)
-            if (
-                isinstance(f_type, type)
-                and issubclass(f_type, AlphaClass)
-                and value is not None
-            ):
+        for fd in dataclasses.fields(self):
+            # field_type = py_lib.get_first_non_none_type(fd.type)
+            value = getattr(self, fd.name)
+            if hasattr(value, "get_fields_dict"):
                 value = value.get_fields_dict()
             else:
                 value = value
-            if keys is not None and f.name not in keys:
+            if keys is not None and fd.name not in keys:
                 continue
-            dct[f.name] = value
+            dct[fd.name] = value
 
         return dct
 
     def compare(
-        self, other, attrs_list: List[str], attrs_eq: Dict[str, str] = None
+        self, other, attrs_list: list[str], attrs_eq: dict[str, str] = None
     ) -> bool:
-        if not isinstance(other, self.__class__):
+        if not isinstance(other, self.__class__) and not issubclass(
+            self.__class__, other.__class__
+        ):
             raise AlphaException(
                 description=f"Cannot compare {self.__class__} object with {other.__class__} object"
             )
@@ -539,7 +614,7 @@ class AlphaDataclass(AlphaClass):
 
         return all(compared)
 
-    def check_attrs_not_none(self, attrs_list: List[str]) -> Optional[bool]:
+    def check_attrs_not_none(self, attrs_list: list[str]) -> bool | None:
         if attrs_list is None:
             return
 
@@ -557,7 +632,7 @@ class AlphaDataclass(AlphaClass):
 
         return True
 
-    def check_attrs_none(self, attrs_list: List[str]) -> Optional[bool]:
+    def check_attrs_none(self, attrs_list: list[str]) -> bool | None:
         if attrs_list is None:
             return
 
@@ -575,7 +650,7 @@ class AlphaDataclass(AlphaClass):
 
         return True
 
-    def replace_empty_with_none(self, attrs_list: List[str]) -> Optional[bool]:
+    def replace_empty_with_none(self, attrs_list: list[str]) -> bool | None:
         if attrs_list is None:
             return
 
@@ -591,11 +666,29 @@ class AlphaDataclass(AlphaClass):
 
         return True
 
+    def check_attributes_max_lenght(self, attrs_list: list[tuple[str, int]]):
+        if attrs_list is None:
+            return
+
+        for attr, max_length in attrs_list:
+            if hasattr(self, attr):
+                self_attr = getattr(self, attr)
+                if self_attr is not None and len(self_attr) > max_length:
+                    raise AlphaException(
+                        description=f"{self.__class__.__name__} {attr=} length={len(self_attr)} is > than {max_length=}"
+                    )
+            else:
+                raise AlphaException(
+                    description=f"{self.__class__.__name__} object has no {attr=}"
+                )
+
+        return True
+
     def update_from_auto_map_from_dict(
         self,
-        dict_values: Dict[str, object],
-        associations: Dict[str, str] = {},
-        no_match: List[str] = [],
+        dict_values: dict[str, object],
+        associations: dict[str, str] = {},
+        no_match: list[str] = [],
         flat: bool = False,
     ):
         instance = self.auto_map_from_dict(
@@ -606,10 +699,13 @@ class AlphaDataclass(AlphaClass):
             if fd is not None:
                 setattr(self, key, fd)
 
-    def flatten(self, parent: str = None):
+    def flatten(self, parent: str | None = None):
         output = {}
         for key, field in self.__dataclass_fields__.items():
-            value = self.__dict__[key]
+            if not key in self.__dict__:
+                value = field.default
+            else:
+                value = self.__dict__[key]
             value_key = f"{parent}.{key}" if parent is not None else key
 
             if value is None:
@@ -632,19 +728,31 @@ class AlphaDataclass(AlphaClass):
         return output
 
     @classmethod
-    def get_fields_names(dataclass_type):
-        return [f.name for f in dataclasses.fields(dataclass_type)]
+    def get_fields_names(dataclass_type, init_only: bool = False):
+        return [
+            f.name
+            for f in dataclasses.fields(dataclass_type)
+            if (init_only and f.init) or not init_only
+        ]
+
+    @classmethod
+    def get_fields(dataclass_type, init_only: bool = False):
+        return [
+            f
+            for f in dataclasses.fields(dataclass_type)
+            if (init_only and f.init) or not init_only
+        ]
 
     @classmethod
     def auto_map_from_dict(
         dataclass_type,
-        dict_values: Dict[str, object],
-        associations: Dict[str, str] = {},
-        no_match: List[str] = [],
+        dict_values: dict[str, object],
+        associations: dict[str, str] = {},
+        no_match: list[str] = [],
         flat: bool = False,
     ):
         global DATACLASS_AUTO_MAP_MATCHS, FORCE_AUTO_MAP
-        if dict_values is None:
+        if dict_values is None or len(dict_values) == 0:
             return dataclass_type()
 
         dataclass_name = str(dataclass_type).split(".")[-1].replace("'>", "")
@@ -652,11 +760,13 @@ class AlphaDataclass(AlphaClass):
 
         # FIELDS
         if not uuid in DATACLASS_AUTO_MAP_MATCHS or FORCE_AUTO_MAP:
-            DATACLASS_AUTO_MAP_MATCHS[uuid] = AutoMapping(
+            auto_mapping = AutoMapping(
                 dataclass_type, associations=associations, no_match=no_match, flat=flat
             )
-
-        fields_values = DATACLASS_AUTO_MAP_MATCHS[uuid].map(dict_values)
+            DATACLASS_AUTO_MAP_MATCHS[uuid] = auto_mapping
+        else: 
+            auto_mapping = DATACLASS_AUTO_MAP_MATCHS[uuid]
+        fields_values = auto_mapping.map(dict_values)
 
         instance = None
         required_fields = [
@@ -671,63 +781,139 @@ class AlphaDataclass(AlphaClass):
         if not is_required_fields:
             return None
         try:
-            instance = dataclass_type(**fields_values)
+            if hasattr(dataclass_type, "map_from_dict"):
+                instance = dataclass_type.map_from_dict(
+                    fields_values, mapping_mode=MappingMode.STRICT
+                )
+            else:
+                fields_values = {
+                    x: y
+                    for x, y in fields_values.items()
+                    if x in dataclass_type.get_fields_names(True)
+                }
+                instance = dataclass_type(**fields_values)
         except Exception as ex:
             print(f"Error mapping {dataclass_type}")
+            tb = traceback.format_exc()
             raise ex
         return instance
 
     @classmethod
-    def map_from_json(dataclass_type, json_values: str, automap: bool = False):
-        dict_value = (
-            json_lib.load_json(json_values)
-            if type(json_values) != dict
-            else json_values
+    def map_from_json(
+        dataclass_type, dict_values: str, mapping_mode: MappingMode | str | None = None
+    ):
+        dict_values = (
+            json_lib.load_json(dict_values)
+            if type(dict_values) != dict
+            else dict_values
         )
         try:
-            obj = dataclass_type.map_from_dict(dict_value, automap=automap)
+            obj = dataclass_type.map_from_dict(dict_values, mapping_mode=mapping_mode)
         except Exception as ex:
-            print(f"Failed to map {dataclass_type} with: {json_values}")
+            print(f"Failed to map {dataclass_type} with: {dict_values}")
             print(ex)
             obj = None
         return obj
 
     @classmethod
     def map_from_dict(
-        dataclass_type, dict_value: Dict[str, object], automap: bool = False
+        dataclass_type,
+        dict_values: dict[str, object] | dict[str, str],
+        mapping_mode: MappingMode | str | None = None,
     ):
-        if automap and hasattr(dataclass_type, "auto_map_from_dict"):
-            return dataclass_type.auto_map_from_dict(dict_values=dict_value)
+        if is_auto_map(dataclass_type, mapping_mode=mapping_mode):
+            return dataclass_type.auto_map_from_dict(dict_values=dict_values)
 
-        if (
-            not py_lib.is_subtype(type(dict_value), Dict)
-            and not py_lib.is_subtype(type(dict_value), dict)
-            and not type(dict_value) == dict
-        ):
+        if not isinstance(dict_values, dict):
             return None
-        if dict_value is None:
+
+        if dict_values is None:
             return None
 
         field_values = {}
-        for key, value in dict_value.items():
-            if key in dataclass_type.__dataclass_fields__:
-                field = dataclass_type.__dataclass_fields__[key]
-                field_type = field.type
+        for key, value in dict_values.items():
+            no_match = False
+            if dataclass_type._map_keys is not None:
+                for ck, regex in dataclass_type._map_keys.items():
+                    if regex.startswith("re:"):
+                        matchs = re.findall(regex[3:], key)
+                        if len(matchs) != 0:
+                            key = ck
+                            break
+                    elif regex == key:
+                        key = ck
+                        break
+            if dataclass_type._no_map is not None:
+                for regex in dataclass_type._no_map:
+                    if regex.startswith("re:"):
+                        matchs = re.findall(regex[3:], key)
+                        if len(matchs) != 0:
+                            no_match = True
+                            break
+                    elif regex == key:
+                        no_match = True
+                        break
+            if no_match:
+                continue
 
-                if py_lib.is_subtype(field_type, List) or py_lib.is_subtype(
-                    field_type, list
-                ):
-                    if value is not None:
-                        field_values[key] = [
-                            convert_value_from_field(
-                                py_lib.get_subtype(field_type), v, automap=automap
-                            )
-                            for v in value
-                        ]
-                    else:
-                        field_values[key] = []
-                else:
-                    field_values[key] = convert_value_from_field(
-                        field_type, value, automap=automap
-                    )
-        return dataclass_type(**field_values)
+            if key not in dataclass_type.__dataclass_fields__:
+                continue
+
+            field = dataclass_type.__dataclass_fields__[key]
+            field_type = py_lib.get_first_non_none_type(field.type)
+
+            if not field.init:
+                continue
+
+            if py_lib.is_subtype(field_type, list):
+                field_values[key] = []
+                if value is not None:
+                    field_values[key] = [
+                        convert_value_from_field(
+                            py_lib.get_subtype(field_type), v, mapping_mode=mapping_mode
+                        )
+                        for v in value
+                    ]
+            else:
+                field_values[key] = convert_value_from_field(
+                    field_type, value, mapping_mode=mapping_mode
+                )
+
+        try:
+            dc = dataclass_type(**field_values)
+        except Exception as ex:
+            dc = None
+            if "missing" in str(ex) and "required" in str(ex):
+                return None
+            raise ex
+        return dc
+
+    @classmethod
+    def map(
+        dataclass_type, dict_values: dict, mapping_mode: MappingMode | str | None = None
+    ):
+        if dataclass_type.get_mapping_mode() is not None and mapping_mode is None:
+            mapping_mode = dataclass_type.get_mapping_mode()
+        elif mapping_mode is None:
+            mapping_mode = MappingMode.STRICT
+
+        mapping_mode_str = str(mapping_mode)
+        if type(mapping_mode) != str:
+            mapping_mode_str: str = str(mapping_mode.value)
+        mapping_mode_str = mapping_mode_str.lower()
+
+        match mapping_mode_str:
+            case MappingMode.AUTO.value:
+                return dataclass_type.auto_map_from_dict(dict_values)
+
+        return dataclass_type.map_from_dict(dict_values, mapping_mode)
+
+
+def is_dataclass(o):
+    if dataclasses.is_dataclass(o):
+        return True
+    if isinstance(o, AlphaDataclass) or (
+        isinstance(o, type) and issubclass(o, AlphaDataclass)
+    ):
+        return True
+    return py_lib.is_subtype(o, AlphaDataclass)
