@@ -1,30 +1,26 @@
-import logging, importlib
-import os, sys, warnings
+# MODULES
+import os, sys, warnings, logging, re
 
-from types import ModuleType
-from typing import List, Dict
-
-from sqlalchemy import MetaData
-
-from ...database.models import AlphaTable
+from sqlalchemy import exc as sqlalchemy_exc
 
 with warnings.catch_warnings():
     from flask_marshmallow import Marshmallow
-from flask_sqlalchemy import DefaultMeta
+from flask_cors import CORS
 
+# MODELS
 from ....models.main import AlphaClass
 from ....models.logger import AlphaLogger, DEFAULT_FORMAT, DEFAUT_DATE_FORMAT
-from ....libs import flask_lib, io_lib
 
 from ....models.config import AlphaConfig
 from ....models.main import AlphaException
 from ...api import AlphaFlask
-from flask_cors import CORS
+
+# LIBS
+from ....libs import io_lib
 
 from ...database.structure import AlphaDatabase
 
-from ....utils.tasks import start_celery
-
+# CONFIGS
 from ....config.main_configuration import CONFIGURATION
 
 
@@ -49,16 +45,16 @@ class AlphaCore(AlphaClass):
 
         self.root_alpha = os.path.dirname(__file__).split("models")[0]
 
-        self.config: AlphaConfig = None
+        self.config: AlphaConfig | None = None
 
         self.initiated: bool = False
-        self.loggers: Dict[str, AlphaLogger] = {}
+        self.loggers: dict[str, AlphaLogger] = {}
 
-        self.ma: Marshmallow = None
-        self._db: AlphaDatabase = None
-        self.api: AlphaFlask = None
+        self.ma: Marshmallow | None = None
+        self._db: AlphaDatabase | None = None
+        self.api: AlphaFlask | None = None
 
-        self._models_sources: List[str] = []
+        self._models_sources: list[str] = []
         self.databases_models: list = []
 
         configuration = (
@@ -66,8 +62,9 @@ class AlphaCore(AlphaClass):
             if not CONFIGURATION.CONFIGURATION_ENV_NAME in os.environ
             else os.environ[CONFIGURATION.CONFIGURATION_ENV_NAME].lower().strip()
         )
-        self.configuration: str = configuration
-        self.configuration_name: str = configuration
+
+        self.configuration: str | None = configuration
+        self.configuration_name: str | None = configuration
 
         self.config: AlphaConfig = AlphaConfig(
             CONFIGURATION.MAIN_CONFIGURATION_NAME,
@@ -81,7 +78,7 @@ class AlphaCore(AlphaClass):
             warnings.filterwarnings("ignore", message=ignored_warning)"""
 
         self.logger_root = self.config.get("directories/logs", required=True)
-        self.log: AlphaLogger = None
+        self.log: AlphaLogger | None = None
 
         self.__set_loggers()
         self.__configure_databases()
@@ -270,7 +267,7 @@ class AlphaCore(AlphaClass):
                     c = f"(DESCRIPTION = (LOAD_BALANCE=on) (FAILOVER=ON) (ADDRESS = (PROTOCOL = TCP)(HOST = {host})(PORT = {port})) (CONNECT_DATA = (SERVER = DEDICATED) (SERVICE_NAME = {name})))"
                 cnx_str = f"oracle://{user}:{password}@{c}"
             elif db_type == "sqlite":
-                cnx_str = "sqlite:///" + cf_db["path"]
+                cnx_str = "sqlite:///" + cf_db["path"]  # + "?timeout=5"
 
             if "ssl" in cf_db and cf_db["ssl"] is not None:
                 if not all(
@@ -294,10 +291,14 @@ class AlphaCore(AlphaClass):
                 cf_db["cnx"] = cnx_str
                 db_cnx[db_name.upper()] = cf_db
 
-        self.db_cnx = db_cnx
+                pattern = re.compile(r":([^@:]+)@")
+                masked_conn_str = re.sub(pattern, ":****@", cnx_str)
+                self.log.debug(
+                    f"Prepared cnx from {masked_conn_str} for bind {db_name.upper()}",
+                    enabled=False,
+                )
 
-    def get_database(self, name):
-        return self._db
+        self.db_cnx = db_cnx
 
     def set_configuration(self, configuration_name):
         if configuration_name is None and self.config.configuration is not None:
@@ -330,9 +331,10 @@ class AlphaCore(AlphaClass):
             root=self.root,
             core=self,
         )
-        # if self.config.get("cors"):
-        # CORS(self.api, resources={r"*": {"origins": "*"}}, supports_credentials=True)
-        # logging.getLogger("flask_cors").level = logging.DEBUG
+
+        # APM
+        self._set_apm_configuration()
+
         self.api.config["CORS_HEADERS"] = "Content-Type"
 
         self.ma = self.api.ma
@@ -365,6 +367,10 @@ class AlphaCore(AlphaClass):
         if db_logger is None:
             db_logger = self.get_logger(CONFIGURATION.MAIN_LOGGER_NAME)
 
+        is_sa_warnings = self.config.get("orm/warnings", False)
+        if not is_sa_warnings:
+            warnings.filterwarnings("ignore", category=sqlalchemy_exc.SAWarning)
+
         self._db = AlphaDatabase(
             self.api,
             name=CONFIGURATION.MAIN_DATABASE_NAME,
@@ -378,11 +384,28 @@ class AlphaCore(AlphaClass):
         """self._db.ensure("tests", drop=True)
         self._db.ensure("files_process")"""
 
-    def get_database(self, name=None) -> AlphaDatabase:
-        self.prepare_api(self.configuration)
+    def _set_apm_configuration(self):
+        apm_config = self.config.get("apm", None)
+        if apm_config is not None and apm_config.get("active", False):
+            from alphaz.utils.apm import ora
+            from elasticapm.contrib.flask import ElasticAPM
+            from elasticapm.handlers.logging import LoggingHandler
 
-        name = name.lower()
-        return self._db
+            """logger = logging.getLogger()
+            console_handler = logging.StreamHandler()
+            console_handler.setLevel(logging.DEBUG)
+            logger.addHandler(console_handler)"""
+
+            apm_config = {x.upper(): y for x, y in apm_config.items()}
+            self.api.config["ELASTIC_APM"] = apm_config
+            self.info(f"Starting apm with configuration {apm_config}...")
+            self.apm = ElasticAPM(self.api)
+
+            # Configurez le gestionnaire LoggingHandler
+            handler = LoggingHandler(self.apm.client, level=logging.DEBUG)
+
+            # Ajoutez le gestionnaire ElasticAPMHandler au journal de Flask
+            self.api.logger.addHandler(handler)
 
     def _check_configuration(self):
         if self.config is None:
@@ -391,108 +414,10 @@ class AlphaCore(AlphaClass):
                 print("ERROR: Configuration need to be initialized")
                 exit()
 
-    def get_table(self, schema: str, table: str):
-        db = self.get_database(schema)
-        return db.get_table(schema=schema, table=table)
-
-    def database_init_all(
-        self,
-        binds: List[str] = None,
-        tables: List[AlphaTable] = None,
-        drop: bool = False,
-        create: bool = False,
-        sqlite: bool = True,
-    ):
-        self.prepare_api(self.configuration)
-        self.load_models_sources()
-
-        if sqlite:
-            binds = [
-                x.upper()
-                for x, y in self.db_cnx.items()
-                if y["type"] == "sqlite" and (binds is None or x in binds)
-            ]
-
-        """for bind, tables_dict in self.tables.items():
-            if binds is None or bind in binds:
-                for table_name, table_model in tables_dict.items():
-                    engine = self.db.get_engine(bind=bind)
-                    if drop:
-                        table_model.__table__.drop(engine)
-                    table_model.__table__.create(engine)"""
-        """if drop:
-            self._db.drop_all(bind=binds, tables=tables)
-        self._db.create_all(bind=binds, tables=tables)"""
-
-        tables_by_bind = {}
-        for table_model in tables:
-            bind = (
-                "MAIN"
-                if not hasattr(table_model, "__bind_key__")
-                else table_model.__bind_key__
-            )
-            if not bind in tables_by_bind:
-                tables_by_bind[bind] = [table_model]
-            else:
-                tables_by_bind[bind].append(table_model)
-
-        for bind, tables_models in tables_by_bind.items():
-            tables = [x.__table__ for x in tables_models]
-            meta = MetaData(self.db.get_engine(bind=bind))
-            if drop:
-                meta.drop_all(tables=tables)
-            meta.create_all(tables=tables)
-
-    def create_table(self, schema: str, table: str, drop: bool = False):
-        db = self.get_database(schema)
-        table_object = self.get_table(schema, table)
-        if drop:
-            try:
-                table_object.__table__.drop(db.engine)
-            except:
-                pass
-        table_object.__table__.create(db.engine)
-        return True
-
-    def drop_table(self, schema: str, table: str):
-        db = self.get_database(schema)
-        table_object = self.get_table(schema, table)
-        table_object.__table__.drop(db.engine)
-
-    @property
-    def models_sources(self):
-        if len(self._models_sources) == 0:
-            self._models_sources = self.config.get("directories/database_models")
-            if not self._models_sources:
-                self.log.error(
-                    f"Missing <directories/database_models> entry in configuration {self.conf.filepath}"
-                )
-                exit()
-        return self._models_sources
-
-    @models_sources.setter
-    def models_sources(self, models_sources):
-        self._models_sources = models_sources
-
-    def load_models_sources(self):
-        if len(self.models_sources) != 0 and len(self.databases_models) != 0:
-            return
-
-        self.log.info(f"Getting models definitions from {self.models_sources}")
-        (
-            self.databases_models,
-            self.tables,
-        ) = flask_lib.get_definitions_modules_and_tables(
-            self.models_sources, log=self.log
-        )
-
-    def get_table_model(self, tablename: str):
-        return flask_lib.get_table_model(tablename, self.models_sources, log=self.log)
-
     def get_data_directory(self):
         return self.config.get("directories/data", required=True)
 
-    def save(self, object_to_save: object, filename: str, ext: str = None):
+    def save(self, object_to_save: object, filename: str, ext: str | None = None):
         filename = (
             filename
             if os.sep in filename
@@ -500,7 +425,7 @@ class AlphaCore(AlphaClass):
         )
         io_lib.archive_object(object_to_save=object_to_save, filename=filename, ext=ext)
 
-    def load(self, filename: str, ext: str = None, default: object = None):
+    def load(self, filename: str, ext: str | None = None, default: object = None):
         filename = (
             filename
             if os.sep in filename
