@@ -1,6 +1,4 @@
-import re
-
-from typing import Dict
+import re, traceback
 
 EXCEPTIONS = {}
 
@@ -9,7 +7,7 @@ def get_message_from_name(name):
     if type(name) != str:
         name = str(name)
     name = name.replace("_", " ")
-    return name[0].upper() + name[1:]
+    return name[0].upper() + name[1:] if len(name) != 0 else name
 
 
 class AlphaException(Exception):
@@ -18,8 +16,9 @@ class AlphaException(Exception):
         name="exception",
         warning: bool = False,
         description=None,
-        parameters: Dict[str, object] = {},
+        parameters: dict[str, object] = {},
         ex: Exception = None,
+        traces_levels: int = 10,
     ):
         self.name = name if name is not None else "exception"
         self.warning = 1 if warning else 0
@@ -35,9 +34,13 @@ class AlphaException(Exception):
             self.description = EXCEPTIONS[name]["text"]
         else:
             self.description = description or get_message_from_name(name)
+
         if ex is not None:
             self.description += f"\nex: {ex}"
 
+        traces = traceback.format_stack()
+        self.traces = [f"{tr}" for tr in traces[-(traces_levels + 1) : -1]]
+
         if len(parameters) != 0 and type(parameters) == dict:
             # parameters = re.findall(r'{[a-zA-Z_-]+',self.description)
             parameters_values = []
