@@ -5,3 +5,5 @@ class Levels(enum.Enum):
     LOW = enum.auto()
     MEDIUM = enum.auto()
     HIGH = enum.auto()
+    CRITICAL = enum.auto()
+    REQUIRED = enum.auto()
