@@ -1,13 +1,18 @@
-import datetime, re
-import typing
+# MODULES
+import datetime
 
-from core import core
+# MODELS
 from ..database.main_definitions import Tests
 
+# LIBS
 from ...libs import number_lib
 
+# LOCAL
 from ._levels import Levels
 
+# CORE
+from core import core
+
 log = core.get_logger("tests")
 
 
@@ -44,11 +49,14 @@ class TestMethod:
         self.end_time: datetime.datetime = None
         self.elapsed: int = 0
         self.last_run_elapsed = None
-        self.coverages: typing.Dict[str, object] = {}
+        self.coverages: dict[str, object] = {}
 
         self.ex: Exception = None
 
-    def test(self, classTest=None, coverage: bool = False):
+    def test(self, classTest=None, coverage: bool = False, failed_only: bool = False):
+        if failed_only and self.status:
+            return self.status
+
         if classTest is None:
             classTest = self.classTest()
 
@@ -70,7 +78,6 @@ class TestMethod:
         except Exception as ex:
             self.ex = ex
             raise ex
-        classTest.end()
 
         log.info(
             f"Function <{self.name}> of <{type(self).__name__}> in category <{self.category}>: {'X' if not self.status else 'O'}"
@@ -94,9 +101,12 @@ class TestMethod:
             end_time=self.end_time,
             elapsed=self.elapsed,
         )
-        core.db.add_or_update(test)
+        try:
+            core.db.add_or_update(test)
+        except:
+            core.log.error(f"Failed to set test state")
 
-    def update_from(self, source):
+    def update_from_database(self, source):
         for key in self.__dict__.keys():
             if hasattr(source, key):
                 source_element = getattr(source, key)
@@ -117,7 +127,7 @@ class TestMethod:
             order_by=Tests.start_time.desc(),
             first=True,
         )
-        self.update_from(test)
+        self.update_from_database(test)
 
     def save(self):
         classTest = self.classTest()
