@@ -1,32 +1,42 @@
+# MODULES
 import traceback, inspect, trace, sys, os
 
 from dataclasses import copy, dataclass
-from core import core
-from typing import List, Dict
-
 import pandas as pd
 
-from ._save import AlphaSave
+# CORE
+from core import core
 
-from alphaz.libs import dict_lib
+# LIBS
+from alphaz.libs import dict_lib, database_lib
+
+# MODELS
+from alphaz.models.main import AlphaDataclass, AlphaClass
 
 LOG = core.get_logger("tests")
 
 
 @dataclass
-class TestInput:
+class TestInput(AlphaDataclass):
     value: str
     number: int
 
 
-class AlphaTest:
+class AlphaTest(AlphaClass):
     _test = True
     category = ""
     index = 0
     current_test_name = ""
+    disable: bool = False
 
-    outputs: Dict[str, bool] = {}
-    coverages: Dict[str, object] = {}
+    outputs: dict[str, bool] = {}
+    coverages: dict[str, object] = {}
+
+    def __init__(self):
+        self.begin_class()
+
+    def begin_class(self):
+        pass
 
     def begin(self):
         pass
@@ -34,9 +44,31 @@ class AlphaTest:
     def end(self):
         pass
 
-    def test(self, name, coverage: bool = False):
+    def init_tables(
+        self,
+        tables: list[str],
+        binds: list[str] | None = None,
+        truncate: bool = False,
+        drop: bool = False,
+        create: bool = False,
+        init: bool = True,
+        init_views: bool = False,
+        no_log: bool = True,
+    ):
+        database_lib.init_databases(
+            core=core,
+            binds=binds,
+            tables=tables,
+            truncate=truncate,
+            drop=drop,
+            create=create,
+            init=init,
+            init_views=init_views,
+            no_log=no_log,
+        )
+
+    def test(self, name, coverage: bool = False) -> bool:
         self.output = None
-        self.partial_output = None
 
         status = False
         self.current_test_name = name
@@ -65,34 +97,27 @@ class AlphaTest:
 
         return status
 
-    def save(self, name):
-        fct = getattr(self, name)
-        if fct is None:
-            return False
+    def reverse(self) -> bool:
+        return self.__reverse_output()
 
-        if inspect.unwrap(fct).__name__ == save_method_name:  # TODO: modify
-            object_to_save = fct(get_return=True)
-            object_name_to_save = fct(get_name=True)
-            AlphaSave.save(object_to_save, object_name_to_save)
-
-    def reverse(self):
-        self.__reverse_output()
-
-    def __reverse_output(self):
+    def __reverse_output(self) -> bool:
         self.output = not self.output
-        self.partial_output = not self.partial_output
         return self.output
 
-    def _set_output(self, status):
+    def _set_output(self, status) -> bool:
         if self.output is not None:
             self.output = status and self.output
         else:
             self.output = status
-        self.partial_output = status
-        return self.partial_output
+        return status
+
+    def _assert(self, status, conditions: list[bool] = [], msg: str = "") -> bool:
+        if len(conditions) != 0 and type(conditions) == list:
+            status = status and all(conditions)
+        return self._set_output(status)
 
     def assert_array_equal(
-        self, a, b, conditions: List[bool] = [], msg: str = ""
+        self, a, b, conditions: list[bool] = [], msg: str = ""
     ) -> bool:
         status = len(a) == len(b)
         if status:
@@ -105,25 +130,20 @@ class AlphaTest:
             )
         return self._assert(status, conditions)
 
-    def _assert(self, status, conditions: List[bool] = [], msg: str = "") -> bool:
-        if len(conditions) != 0 and type(conditions) == list:
-            status = status and all(conditions)
-        return self._set_output(status)
-
-    def assert_is_not_none(self, a, conditions: List[bool] = [], msg: str = "") -> bool:
+    def assert_is_not_none(self, a, conditions: list[bool] = [], msg: str = "") -> bool:
         status = a is not None
         if not status:
             LOG.info(f"{self.current_test_name} {msg} - Assert: value is None")
         return self._assert(status, conditions)
 
-    def assert_is_none(self, a, conditions: List[bool] = [], msg: str = "") -> bool:
+    def assert_is_none(self, a, conditions: list[bool] = [], msg: str = "") -> bool:
         status = a is None
         if not status:
             LOG.info(f"{self.current_test_name} {msg} - Assert: value is not None")
         return self._assert(status, conditions)
 
     def assert_is_empty(
-        self, a, conditions: List[bool] = [], msg: str = "", attribute=None
+        self, a, conditions: list[bool] = [], msg: str = "", attribute=None
     ) -> bool:
         status = self.assert_is_not_empty(
             a, conditions=conditions, attribute=attribute, msg=msg
@@ -133,16 +153,16 @@ class AlphaTest:
         return self.__reverse_output()
 
     def assert_are_keys_in_model_attributes(
-        self, a, model, attributes=[], conditions: List[bool] = [], msg: str = ""
-    ):
+        self, a, model, attributes=[], conditions: list[bool] = [], msg: str = ""
+    ) -> bool:
         status = self.assert_are_keys_in_models_attributes(
             a, [model], attributes=attributes, conditions=conditions
         )
         return status
 
     def assert_are_keys_in_models_attributes(
-        self, a, models, attributes=[], conditions: List[bool] = [], msg: str = ""
-    ):
+        self, a, models, attributes=[], conditions: list[bool] = [], msg: str = ""
+    ) -> bool:
         self.assert_is_not_none(a, msg=msg)
         for model in models:
             attributes.extend(list(model.get_schema()._declared_fields.keys()))
@@ -156,7 +176,7 @@ class AlphaTest:
         return self._assert(status, conditions)
 
     def assert_has_model_attributes(
-        self, a, model, conditions: List[bool] = [], msg: str = ""
+        self, a, model, conditions: list[bool] = [], msg: str = ""
     ) -> bool:
         self.assert_is_not_none(a, msg=msg)
         fields = list(model.get_schema()._declared_fields.keys())
@@ -170,21 +190,23 @@ class AlphaTest:
             )
         return self._assert(status, conditions)
 
-    def assert_is_true(self, a, conditions: List[bool] = [], msg: str = ""):
+    def assert_is_true(
+        self, a, conditions: list[bool] = [], msg: str = "", no_log: bool = False
+    ) -> bool:
         status = a
-        if not status:
+        if not status and not no_log:
             LOG.info(f"{self.current_test_name} {msg} - Assert: value {a} is not True")
         return self._assert(status, conditions)
 
-    def assert_is_false(self, a, conditions: List[bool] = [], msg: str = ""):
+    def assert_is_false(self, a, conditions: list[bool] = [], msg: str = "") -> bool:
         status = a
         if status:
             LOG.info(f"{self.current_test_name} {msg} - Assert: value {a} is not False")
         return self._assert(not status, conditions)
 
     def assert_contains(
-        self, container, element, conditions: List[bool] = [], msg: str = ""
-    ):
+        self, container, element, conditions: list[bool] = [], msg: str = ""
+    ) -> bool:
         status = element in container
         if not status:
             LOG.info(
@@ -193,7 +215,7 @@ class AlphaTest:
         return self._assert(status, conditions)
 
     def assert_is_not_empty(
-        self, a, conditions: List[bool] = [], msg: str = "", attribute=None
+        self, a, conditions: list[bool] = [], msg: str = "", attribute=None
     ) -> bool:
         if attribute is not None:
             if not hasattr(a, attribute):
@@ -209,13 +231,19 @@ class AlphaTest:
             LOG.info(f"{self.current_test_name} {msg} - Assert: value is None")
         return self._assert(status, conditions)
 
-    def assert_is_dict(self, a, conditions: List[bool] = [], msg: str = ""):
+    def assert_is_dict(self, a, conditions: list[bool] = [], msg: str = "") -> bool:
         status = type(a) == dict
         if not type(a) == dict:
             LOG.info(f"{self.current_test_name} {msg} - Assert: value is not a dict")
         return self._assert(status, conditions)
 
-    def assert_is_dict_not_empty(self, a, conditions: List[bool] = [], msg: str = ""):
+    def assert_is_list(self, a, conditions: list[bool] = [], msg: str = "") -> bool:
+        status = type(a) == list
+        if not type(a) == list:
+            LOG.info(f"{self.current_test_name} {msg} - Assert: value is not a list")
+        return self._assert(status, conditions)
+
+    def assert_is_dict_not_empty(self, a, conditions: list[bool] = [], msg: str = ""):
         self.assert_is_dict(a, msg=msg)
         status = self.assert_is_not_empty(a, msg=msg)
         if not status:
@@ -223,7 +251,7 @@ class AlphaTest:
         return self._assert(status, conditions)
 
     def assert_is_dict_with_key(
-        self, a, key, conditions: List[bool] = [], msg: str = ""
+        self, a, key, conditions: list[bool] = [], msg: str = ""
     ) -> bool:
         self.assert_is_dict(a, msg=msg)
         status = key in a
@@ -232,7 +260,7 @@ class AlphaTest:
         return self._assert(status, conditions)
 
     def assert_is_dict_with_key_not_empty(
-        self, a, key, conditions: List[bool] = [], msg: str = ""
+        self, a, key, conditions: list[bool] = [], msg: str = ""
     ) -> bool:
         self.assert_is_not_none(a, msg=msg)
         self.assert_is_dict_with_key(a, key, msg=msg)
@@ -244,7 +272,7 @@ class AlphaTest:
         return self._assert(status, conditions)
 
     def assert_is_dict_with_key_with_value(
-        self, a, key, value, conditions: List[bool] = [], msg: str = ""
+        self, a, key, value, conditions: list[bool] = [], msg: str = ""
     ) -> bool:
         status = self.assert_is_dict_with_key_not_empty(a, key, msg=msg)
         if status:
@@ -255,23 +283,60 @@ class AlphaTest:
             )
         return self._assert(status, conditions)
 
+    def assert_is_list_no_empty(self, a, conditions: list[bool] = [], msg: str = ""):
+        status = self.assert_is_list(a, msg=msg)
+        if status:
+            status = self.assert_is_not_empty(a)
+        return self._assert(status, conditions)
+
+    def assert_is_equal(
+        self,
+        a,
+        b,
+        ignore: list | dict | None = None,
+        conditions: list[bool] = [],
+        msg: str = "",
+    ):
+        return self.assert_equal(a, b, ignore=ignore, conditions=conditions, msg=msg)
+
     def assert_equal(
         self,
         a,
         b,
-        ignore: list | dict = None,
-        conditions: List[bool] = [],
+        ignore: list | dict | None = None,
+        conditions: list[bool] = [],
         msg: str = "",
     ) -> bool:
-        if type(a) == dict and type(b) == dict:
+        if isinstance(a, list) and isinstance(b, list) and len(a) == 0 and len(b) == 0:
+            return True
+        if (
+            isinstance(a, list)
+            and isinstance(b, list)
+            and all(isinstance(item, dict) for item in a)
+            and all(isinstance(item, dict) for item in b)
+        ):
+            return all(
+                [
+                    self.assert_equal(
+                        a=item,
+                        b=b[index],
+                        ignore=ignore,
+                        conditions=conditions,
+                        msg=msg,
+                    )
+                    for index, item in enumerate(a)
+                ]
+            )
+
+        elif type(a) == dict and type(b) == dict:
             diff = dict_lib.compare_dicts(a, b, ignore=ignore)
 
             status = diff is None
             if not status:
                 LOG.info(
-                    f"{self.current_test_name} {msg} - Assert: dict are not equals\n"
+                    f"{self.current_test_name} {msg} - Assert: dict are not equals\n\n{dict_lib.show_dict(diff)}"
                 )
-                print("\n", dict_lib.show_dict(diff))
+
         else:
             status = a == b
             if not status:
@@ -280,12 +345,55 @@ class AlphaTest:
                 )
         return self._assert(status, conditions)
 
+    def assert_in_str(
+        self,
+        a: str,
+        b: str,
+        conditions: list[bool] = [],
+        msg: str = "",
+    ) -> bool:
+        for e in [a, b]:
+            if type(e) != str:
+                LOG.info(
+                    f"{self.current_test_name} {msg} - Assert: {e} type is not str"
+                )
+                return self._assert(False, conditions)
+        status = a in b
+        if not status:
+            LOG.info(
+                f"{self.current_test_name} {msg} - Assert: {b} does not include {b}"
+            )
+        return self._assert(status, conditions)
+
+    def assert_list_str_in_str(
+        self,
+        a: list[str],
+        b: str,
+        conditions: list[bool] = [],
+        msg: str = "",
+    ) -> bool:  # TODO add check on a type
+        if type(b) != str:
+            LOG.info(f"{self.current_test_name} {msg} - Assert: {b} type is not str")
+            return self._assert(False, conditions)
+
+        status = True
+        for e in a:
+            if e not in b:
+                status = False
+                LOG.info(
+                    f"{self.current_test_name} {msg} - Assert: {e} does not include in {a}"
+                )
+
+                break
+
+        return self._assert(status, conditions)
+
     def assert_not_equal(
         self,
         a,
         b,
-        ignore: list | dict = None,
-        conditions: List[bool] = [],
+        ignore: list | dict | None = None,
+        conditions: list[bool] = [],
         msg: str = "",
     ) -> bool:
         status = a != b
@@ -297,7 +405,7 @@ class AlphaTest:
         self,
         a,
         length,
-        conditions: List[bool] = [],
+        conditions: list[bool] = [],
         msg: str = "",
         strict: bool = False,
     ) -> bool:
@@ -312,7 +420,7 @@ class AlphaTest:
         return self._assert(status, conditions)
 
     def assert_transaction(
-        self, tr, conditions: List[bool] = [], msg: str = ""
+        self, tr, conditions: list[bool] = [], msg: str = ""
     ) -> bool:
         status_not_none = self.assert_is_not_none(tr, msg=msg)
         status = tr is not None and tr != "timeout"
@@ -330,30 +438,37 @@ class AlphaTest:
         default_values: dict,
         params: dict = {},
         msg: str = "",
-    ):
+        strict: bool = True,
+    ) -> bool:
         raw_params = copy.copy(params)
         from ...libs import api_lib
 
         answer = api_lib.get_api_answer(
-            url=url, method=method, params=params, no_log=True
+            url=url, method=method, params=params, no_log=False
         )
 
-        self.partial_output = True
-        real_output = {}
+        expected_output_diff, real_output = {}, {}
         if not self.assert_equal(answer.error, 0):
             LOG.info("Answer is invalid")
+            return self.output
         for x, y in answer.data.items():
             if x in expected_output:
-                self.assert_equal(y, expected_output[x])
-                real_output[x] = y
+                if not self.assert_equal(y, expected_output[x]):
+                    real_output[x] = y
+                    expected_output_diff[x] = expected_output[x]
             elif x in default_values:
-                self.assert_equal(y, default_values[x])
-            else:
-                self.assert_is_none(y)
-        if not self.partial_output:
+                if not self.assert_equal(y, default_values[x]):
+                    real_output[x] = y
+                    expected_output_diff[x] = default_values[x]
+            elif strict:
+                if not self.assert_is_none(y):
+                    real_output[x] = y
+                    expected_output_diff[x] = None
+        if not self.output:
             LOG.info(
-                f"{self.current_test_name} {msg} - Assert: {raw_params=} > {params=} does not provide an {expected_output=} but {real_output}"
+                f"{self.current_test_name} {msg} - Assert: {raw_params=} > {params=} - discrepancy between:\n\nExpected output:\n\n{expected_output_diff}\n\nReal output:\n\n{real_output}\n"
             )
+        return self.output
 
     def assert_api_answer_fail(
         self,
@@ -361,21 +476,20 @@ class AlphaTest:
         method: str,
         params: dict,
         expected_output: str,
-        default_values: dict,
+        default_values: dict | None = None,
         msg: str = "",
-    ):
+    ) -> bool:
         from ...libs import api_lib
 
         answer = api_lib.get_api_answer(
             url=url, method=method, params=params, no_log=True
         )
 
-        self.partial_output = True
         self.assert_equal(answer.error, 1)
-        self.assert_equal(answer.status, expected_output)
+        self.assert_in_str(expected_output, answer.status_description)
 
-        if not self.partial_output:
+        if not self.output:
             LOG.info(
                 f"{self.current_test_name} {msg} - Assert: {params=} does not provide an {expected_output=} but {answer.status}"
             )
-
+        return self.output
