@@ -1,7 +1,7 @@
 import copy
 import traceback
-import typing
-from typing import List
+
+
 from flask import (
     jsonify,
     make_response,
@@ -12,13 +12,13 @@ from flask import (
     url_for,
     render_template,
 )
+from enum import Enum
 
 from ..config.main_configuration import CONFIGURATION
 
 from ..models.main import AlphaException
 from ..models.api import Parameter, ParameterMode, AlphaRequest
 from ..models.api._route import Route
-from ..models.tests._levels import Levels
 
 from core import core
 
@@ -27,6 +27,8 @@ API = core.api
 DB = core.db
 LOG = core.get_logger("api")
 
+ROUTES_DISABLED = core.config.get("api/routes_disabled", [])
+
 ROUTES = {}
 
 # Specify the debug panels you want
@@ -60,7 +62,7 @@ default_parameters = [
 default_parameters_names = [p.name for p in default_parameters]
 
 
-def get_pagination_parameters(pagination: int = None):
+def get_pagination_parameters(pagination: int | None = None):
     if type(pagination) == bool:
         return [
             Parameter("page", ptype=int, default=0),
@@ -74,8 +76,8 @@ def get_pagination_parameters(pagination: int = None):
 
 
 def _process_parameters(
-    path: str, parameters: List[Parameter], pagination: int = None
-) -> List[Parameter]:
+    path: str, parameters: list[Parameter], pagination: int | None = None
+) -> list[Parameter]:
     parameters = [] if parameters is None else parameters
     overrides = []
     for i, parameter in enumerate(parameters):
@@ -86,7 +88,7 @@ def _process_parameters(
             if parameter.override:
                 overrides.append(parameter.name)
                 continue
-            LOG.critical(
+            LOG.error(
                 f"Parameter could not be named <{parameter.name}> for route <{path}>!"
             )
             exit()
@@ -108,20 +110,23 @@ def _process_parameters(
 
 def route(
     path,
-    parameters: typing.List[Parameter] = None,
+    parameters: list[Parameter] | None = None,
     methods=["GET"],
     cache: bool = False,
     logged: bool = False,
     admin: bool = False,
-    timeout: int = None,
-    category: str = None,
-    description: str = None,
-    mode: str = None,
+    timeout: int | None = None,
+    category: str | None = None,
+    description: str | None = None,
+    mode: str | None = None,
     route_log: bool = True,
-    access_strings: typing.List[str] = [],
-    pagination: int = None,
+    access_strings: list[str] | list[Enum] = [],
+    access_error_message: str | None = None,
+    pagination: int | None = None,
 ):
     path = "/" + path if path[0] != "/" else path
+    if path in ROUTES_DISABLED:
+        return lambda x: x
 
     parameters = _process_parameters(path, parameters, pagination=pagination)
 
@@ -156,7 +161,8 @@ def route(
                 )
 
             if __route.ex is not None:
-                LOG.error(ex=__route.ex)
+                if not __route.no_log:
+                    LOG.error(ex=__route.ex)
                 return __route.get_return()
 
             # check permissions
@@ -167,28 +173,53 @@ def route(
                     else api.admin_token
                 )
                 user = api.get_logged_user()
+                status = api.get_status()
 
                 if logged and user is None and token is None:
                     error_msg = "Wrong permission: empty token"
                     if not __route.no_log:
                         LOG.warning(error_msg)
-                    # __route.set_error_not_logged()
-                    # return __route.get_return()
-                    abort(make_response(jsonify(message=error_msg, error=True), 401))
+                    __route.set_error(
+                        status="wrong_access", description=error_msg, error_code=401
+                    )
+                    return __route.get_return()
+                elif logged and user is None and status[0] == "wrong_token":
+                    error_msg = "Wrong permission: wrong token"
+                    if not __route.no_log:
+                        LOG.warning(error_msg)
+                    __route.set_error(
+                        status="wrong_token", description=error_msg, error_code=401
+                    )
+                    return __route.get_return()
                 elif logged and user is None:
                     error_msg = f"Wrong permission: {user=} not logged"
                     if not __route.no_log:
                         LOG.warning(error_msg)
-                    # __route.set_error_not_logged()
-                    # return __route.get_return()
-                    abort(make_response(jsonify(message=error_msg), 401))
+                    __route.set_error(
+                        status="wrong_access", description=error_msg, error_code=401
+                    )
+                    return __route.get_return()
                 if len(access_strings) != 0:
-                    access_strings.append(CONFIGURATION.SUPER_USER_PERMISSION)
+                    new_access_strings = [
+                        (
+                            access_string.value
+                            if isinstance(access_string, Enum)
+                            else access_string
+                        )
+                        for access_string in access_strings
+                    ]
+
                     is_valid_access = any(
-                        [x in user.permissions for x in access_strings]
+                        [
+                            x in user.permissions
+                            for x in [
+                                *new_access_strings,
+                                *[CONFIGURATION.SUPER_USER_PERMISSION],
+                            ]
+                        ]
                     )
                     if not is_valid_access:
-                        __route.access_denied()
+                        __route.access_denied(access_error_message)
                         return __route.get_return()
             if admin and not api.is_admin():
                 __route.access_denied()
@@ -212,13 +243,13 @@ def route(
                 except Exception as ex:
                     error_format = api.get("error_format")
                     if error_format and error_format.lower() == "exception":
-                        raise __route.set_error(ex)
+                        raise __route.set_error(ex=ex)
                     if not "alpha" in str(type(ex)).lower():
                         if not __route.no_log:
                             LOG.error(ex)
-                        __route.set_error(AlphaException(ex))
+                        __route.set_error(ex=AlphaException(ex))
                     else:
-                        __route.set_error(ex)
+                        __route.set_error(ex=ex)
                 if __route.cache:
                     __route.set_cache()
 
@@ -291,6 +322,26 @@ def route(
 ##################################################################################################################
 # BASE API FUNCTIONS
 ##################################################################################################################
+CORS_CONFIG = None
+
+
+@api.before_request
+def before_request():
+    global CORS_CONFIG
+    if CORS_CONFIG is None:
+        CORS_CONFIG = core.config.get("cors", {})
+    if not "before" in CORS_CONFIG:
+        return
+    if "origins_allowed" in CORS_CONFIG["before"]:
+        allowed_origins = CORS_CONFIG["before"]["origins_allowed"]
+        origin = request.headers.get("Origin")
+        if origin is not None and origin not in allowed_origins:
+            abort(403)  # Forbidden access
+    if "host_allowed" in CORS_CONFIG["before"]:
+        allowed_origins = CORS_CONFIG["before"]["host_allowed"]
+        origin = request.headers.get("Host")
+        if origin is not None and origin not in allowed_origins:
+            abort(403)  # Forbidden access
 
 
 @api.after_request
@@ -298,9 +349,9 @@ def after_request(response):
     response.headers.add("Access-Control-Allow-Origin", "*")
     response.headers.add(
         "Access-Control-Allow-Headers",
-        "Origin,Accept,X-Requested-With,Content-Type,Authorization",
+        "*",
     )
-    response.headers.add("Access-Control-Allow-Methods", "GET,PUT,POST,DELETE,OPTIONS")
+    response.headers.add("Access-Control-Allow-Methods", "*")
     # response.headers.set('Allow', 'GET, PUT, POST, DELETE, OPTIONS')
     response.headers.add("Access-Control-Allow-Credentials", "true")
     return response
